function fluxrope, time, param=p, length=length, debug = debug

if data_type(p) ne 8 then begin
  pp = {sw_param, vsw:[-500.,0.,0.], midtime:0.d, trange:[0.d,0.d], np:0 }
  if keyword_set(time) then begin
    pp.midtime = average(time)
    pp.trange = minmax_range(time)
    pp.np = n_elements(time)
  endif
  p = {cloud_param,theta:10.d, phi:270.d, B0:20.d, R0:.125d, cad:0.d, t0:0.d, pp:pp}
endif

if not keyword_set(time) then return,0

;pp.midtime = average(time)
;pp.trange = minmax_range(time)
;pp.np = n_elements(time)

au = 150e6
alpha = 2.4/p.r0

vsw = p.pp.vsw

st = sin(p.theta  * !dpi / 180.d)
ct = cos(p.theta  * !dpi / 180.d)
sp = sin(p.phi    * !dpi / 180.d)
cp = cos(p.phi    * !dpi / 180.d)

s1 = [st*cp , st*sp,  -ct]
s2 = [-sp   ,    cp,  0.d]
s3 = [ct*cp , ct*sp,  st ]


Rot = [[s1],[s2],[s3]]

Rot_i = transpose(Rot)

nt = n_elements(time)

cadir = crossp(s3,vsw)
cadir = cadir/sqrt(total(cadir^2))

r =  replicate(1.,nt) # (cadir * p.cad) - (time-(p.pp.midtime+p.t0)) # (vsw/au)
rp = Rot ## r

rho = sqrt(rp(*,0)^2 + rp(*,1)^2)

Ba = p.B0 * beselj(alpha*rho,0)
Bt = p.B0 * beselj(alpha*rho,1)


if arg_present(length) then begin
;linear=1
if keyword_set(linear) then begin
  z = Bt/Ba
  rz = sqrt(1+z^2)
  length = ( z*rz + alog(z+rz) ) /2 / z
endif else begin
message,/info,'Determining length for sin '
  z = Bt/Ba
  length = z
  x = dgen(50,range=[0,!pi/2])
  for i=0,n_elements(z)-1 do $
     length(i) = int_tabulated(x,sqrt(1+(z(i)*sin(x))^2))
  length = length *2/!pi
endelse
;length = sqrt(1+ (Bt/Ba)^2)

endif



Bp = [[-rp(*,1)/rho * Bt],[ rp(*,0)/rho * Bt],[ Ba]]

bgse = Rot_i ## Bp

if keyword_set(debug) then stop

return,bgse
end
