pro get_3d_spectra,source, $
  cdf_time=cdf_time,debug=debug,resolution=res,bkgrate=bkgr, $
  pad3=pad3,trange=trange,time2=time2
;load_3dp_data,/if_needed

; source = 'eh'   
; source = 'sf'  
 
if source eq 'eh' then begin
  n_e = 15
  n_b = 88
  numpa = 8
  n_d = 9
  dets= [1,2,3,4,5,6,7,8]
  pdname = source+'pd'
endif
if source eq 'sf' then begin
  n_e = 7
  n_b = 48
  numpa = 8
  n_d = 7
  dets= [1,2,3,4,5,6]
  pdname = source+'pd'
endif

if source eq 'eh' then times = get_eh(/times)
if source eq 'sf' then times = get_sf(/times)
message,/info,'Collecting data'

ntimes = n_elements(times) * keyword_set(times)
data   = fltarr(ntimes,n_e,n_b)
energy = fltarr(ntimes,n_e,n_b)
dt     = fltarr(ntimes,n_e,n_b)
conv   = fltarr(ntimes,n_e,n_b)
valid  = intarr(ntimes)
for i=0l,ntimes-1 do begin
  if source eq 'eh' then d3d = get_eh(index=i)
  if source eq 'sf' then d3d = get_sf(index=i,/chanmode)
  valid[i] = d3d.valid
  data[i,*,*] = d3d.data
  dt[i,*,*]   = d3d.dt
  energy[i,*,*] = d3d.energy
  if source eq 'eh' then  conv[i,*,*] =  d3d.dt * d3d.energy * d3d.gf  * d3d.geomfactor
  if source eq 'sf' then  conv[i,*,*] =  d3d.dt * d3d.denergy * d3d.gf * d3d.feff * 0.3
endfor

w = where(valid,c)
if c eq 0 then message,'No '+source+' data!'
if source eq 'eh' then d3d = get_eh(index=w[0])
if source eq 'sf' then d3d = get_sf(index=w[0],/chanmode)

phi = reform(d3d.phi[0,*])
theta = reform(d3d.theta[0,*])
nrg = reform(d3d.energy[*,0])
sphere_to_cart,1,theta,phi,x,y,z
load_if_needed,'wi_B3'
b = data_cut('wi_B3',times)
bdir = b / ( sqrt(total(b^2,2)) # [1,1,1] )
cospa = bdir[*,0] # x + bdir[*,1] # y + bdir[*,2] # z 

if source eq 'eh' then begin
  det = fix((theta + 90)/22.5 ) + 1
  sunbins = abs(sind(phi)) lt sqrt(2)/2
  okind = where(sunbins eq 0)
  det[where(sunbins)] = 0
  f0 = where(det eq 0)
  f1 = where(det eq 1)
  f2 = where(det eq 2)
  f3 = where(det eq 3)
  f4 = where(det eq 4)
  f5 = where(det eq 5)
  f6 = where(det eq 6)
  f7 = where(det eq 7)
  f8 = where(det eq 8)
  binmap = [f0,f1,f2,f3,f4,f5,f6,f7,f8]
endif

if source eq 'sf' then begin
  det = d3d.detector
  sunbins = [8,9,15,31,32,33]
  det[sunbins] = 0
  f0 = where(det eq 0)
  f1 = where(det eq 1)
  f2 = where(det eq 2)
  f3 = where(det eq 3)
  f4 = where(det eq 4)
  f5 = where(det eq 5)
  f6 = where(det eq 6)
  binmap = [f4,f3,f5,f6,f1,f2,f0]
endif
w = where(valid eq 0,c)
if c ne 0 then data[w,*,*] = !values.f_nan

w = where(conv eq 0,c)
if c ne 0 then data[w] = !values.f_nan
flux = data/conv

;  remove spikes
aflux = average(flux,3,stdev=sflux) 
nbad = 0
bad = where(flux[*] gt (aflux[*] # replicate(n_b/4,n_b)) ,nbad)
if nbad ne 0 then data[bad] = !values.f_nan


if n_elements(res) eq 0 then res = 600d

e = 1
flx = time_average(times, reform(flux[*,e,binmap]),res=res,newtime=time2)
name = strcompress(/remov,string(source+'3d',e))
store_data,name+'_bins',data={x:time2,y:flx,v:findgen(n_b)}, $
  dlim={spec:1,no_interp:1,zlog:1,colors:det[binmap] mod 6 +1}
store_data,name+'_foo',data=name+'_bins',dlim={spec:0,ylog:1}

if 0 then begin
  ctime,t
  w = where(times lt t[1] and times gt t[0])
  bkgr = average(data[w,*,*],1,/nan)
  eb = [1,2,3]
  bkgr[*,*] = replicate(1,n_e) # average(bkgr[eb,*],1)

endif 
if not keyword_set(bkgr) then  bkgr = replicate(0.,n_e,n_b)


pab = fix( acos(cospa)/!pi * numpa )
pa = (findgen(numpa)+.5)/numpa   * 180


message,/info,'Doing pads'
tdata = fltarr(ntimes,n_e,numpa,n_d)
tconv = fltarr(ntimes,n_e,numpa,n_d)
for d=0l,n_d-1 do begin
 bins = where(det eq d,nbins)
 if nbins eq 0 then continue
 for e=0l,n_e-1 do begin
  for b=0l,nbins-1 do begin
    dat = flux[*,e,bins[b]]
    for p=0l,numpa-1 do begin
       w = where(pab[*,bins[b]] eq p,c)
       if c ne 0 then begin
         tdata[w,e,p,d] = tdata[w,e,p,d] + data[w,e,bins[b]] - bkgr[e,bins[b]] * dt[w,e,bins[b]]
         tconv[w,e,p,d] = tconv[w,e,p,d] + conv[w,e,bins[b]]
       endif else print,e,d,bins[b],p
    endfor
  endfor
 endfor
endfor

; end
 
tdata1 = total( tdata[*,*,*,dets],4 ,/nan)
tconv1 = total( tconv[*,*,*,dets],4 ,/nan)

pad1 = tdata1/tconv1
if nbad ne 0 then pad1[bad,*,*] =!values.f_nan
 
pad3 = time_average( times, pad1, res=res, newtime=time2,trange=trange )
 
e=1
for e=1,n_e-1 do begin
  name = strcompress(/remo,string(/print,pdname,e,'d'))
  ytitle = strtrim(round(nrg[e]),2)
  store_data,name,data={x:time2, y:reform(pad3[*,e,*]),v:pa},dlim={no_interp:1,spec:1,ytitle:ytitle,yrange:[0,180],ystyle:1}
endfor

wght = sind(pa)/total(sind(pa))
flx3 = 0
for p=0,numpa-1 do flx3 = flx3 +  pad3[*,*,p] * wght[p]
name = source+'flx'
store_data,name,data={x:time2, y:flx3},dlim={ylog:1}

if keyword_set(debug) then stop

end

