pro get_ehsp_spectra,cdf_time=cdf_time

if keyword_set(cdf_time) then begin
    suffix = '_v02'
    datelab = time_string(cdf_time[0],format=2,/date_only)+suffix
    fname = full_pathname('wi_3dp_ehsp_'+datelab,/mkdir)
    if file_test(/regular,fname+'.cdf') then return
endif

load_3dp_data,/if_needed

units = 'flux'
  
if  not keyword_set(tnames('ehsp_44'))  then begin
  ehbins = bytarr(88)
  ehbins[22:65] = 1
  get_spec,units=units,'eh',bins=ehbins,name = 'ehsp_44'
endif

if keyword_set(cdf_time) then begin
    tplot2cdf,'ehsp_44',filename=fname,trange=cdf_time  
endif

end





pro get_sfsp_spectra,cdf_time=cdf_time

if keyword_set(cdf_time) then begin
    suffix = '_v02'
    datelab = time_string(cdf_time[0],format=2,/date_only)+suffix
    fname = full_pathname('wi_3dp_sfsp_'+datelab,/mkdir)
    if file_test(/regular,fname+'.cdf') then return
endif

load_3dp_data,/if_needed

units = 'flux'
  
  sfhbins =replicate(1b,48)
  sflbins = sfhbins * 0b
  lowbins = [20,21,22,23,44,45,46,47]
  sunbins = [7,8,9,15,31,32,33]
  sfhbins[ sunbins ] = 0    ; Sun anti-sun
  sfhbins[ lowbins ] = 0    ; low gf
  sflbins[ lowbins ] = 1

if not keyword_set(tnames('sfsp_hg')) then $
  get_spec,units=units,'sf',bins=sfhbins,name = 'sfsp_hg'
if not keyword_set(tnames('sfsp_lg')) then $
  get_spec,units=units,'sf',bins=sflbins,name = 'sfsp_lg'

if keyword_set(cdf_time) then begin
    tplot2cdf,'sfsp_hg sfsp_lg',filename=fname,trange=cdf_time
endif

end





pro get_sosp_spectra,cdf_time=cdf_time

if keyword_set(cdf_time) then begin
    suffix = '_v02'
    datelab = time_string(cdf_time[0],format=2,/date_only)+suffix
    fname = full_pathname('wi_3dp_sosp_'+datelab,/mkdir)
    if file_test(/regular,fname+'.cdf') then return
endif

load_3dp_data,/if_needed

units = 'flux'
  sohbins =replicate(1b,48)
  sohbins[ [7,8,9,31,32,33] ] = 0    ; Sun anti-sun
  sohbins[ [20,21,22,23,44,45,46,47] ] = 0    ; low gf
  solbins=replicate(0b,48)
  solbins[ [20,21,22,23,44,45,46,47] ] =1
  
if not keyword_set(tnames('sosp_hg')) then $
  get_spec,units=units,'so',bins=sohbins,name = 'sosp_hg'
if not keyword_set(tnames('sosp_lg')) then $
  get_spec,units=units,'so',bins=solbins,name = 'sosp_lg'
  

if keyword_set(cdf_time) then begin
    tplot2cdf,'sosp_hg sosp_lg',filename=fname,trange=cdf_time,/over
endif

end





pro get_all_spectra,cdf_time=cdf_time

get_ehsp_spectra,cdf_time=cdf_time
get_sfsp_spectra,cdf_time=cdf_time
get_sosp_spectra,cdf_time=cdf_time

end
