pro get_bspec,dname,trange=trange,bkg=bkg,npad=npad, $
   velname=velname,magname=magname,units=units

times = call_function('get_'+dname,/time)
n = n_elements(times)

if keyword_set(trange) then begin
  w = where(times ge trange[0] and times lt trange[1],n)
  if n eq 0 then message,'No data'
  times = times[w]
endif else w = lindgen(n)

n = n_elements(w)

if not keyword_set(magname) then magname='wi_B3'
magf = data_cut(magname,times)

if keyword_set(velname) then vel = data_cut(velname,times)
if keyword_set(scpname) then sc_pot = data_cut(scpname,times)

if n_elements(npad) eq 0 then npad=8
np = keyword_set(npad) ? npad : 1

dat = call_function('get_'+dname,index=w[0])
data = replicate(!values.f_nan,n,dat.nenergy,np)

if not keyword_set(units) then units='flux'

for i=0l,n-1 do begin
  dat = call_function('get_'+dname,index=w[i])
  dat.magf = magf[i,*]
  add_ddata,dat
  if keyword_set(bkg) then dat = sub3d(dat,bkg,sig=2)
  if keyword_set(sc_pot) then dat.sc_pot = sc_pot[i] else dat.sc_pot=8.
  if keyword_set(vel) then begin
    dat.vsw = vel[i,*]
    dat = convert_vframe(dat,/int)
  endif
  dat = conv_units(dat,units)
  bdir = reform(magf[i,*])
  if finite(total(bdir)) then begin
    bdir = bdir / sqrt(total(bdir^2))
    z = sin(dat.theta/!radeg)
    x = cos(dat.theta/!radeg) *cos(dat.phi/!radeg)
    y = cos(dat.theta/!radeg) *sin(dat.phi/!radeg)
  
    pa = acos(x*bdir[0] + y*bdir[1] + z*bdir[2]) * !radeg
    pa = average(pa,1)
    if keyword_set(npad) then begin
      h = histbins(pa,pab,range=[0,180],nbins=npad,reverse=ri)
      whn0 = where(h ne 0,count)
      for j=0,count-1 do begin
         k = whn0[j]
         ind = ri[ ri[k]: ri[k+1]-1 ]
    ;     wght = 1/dat.ddata[*,ind]^2
         data[i,*,k] = average(dat.data[*,ind],2,stdev=s,nan=1,weight=wght)
       ;  std[i,*,k] = s
      endfor
    endif else begin
      s = sort(pa)
      data[i,*] = average(dat.data[*,s[0:3]],2)
    endelse
  endif
  print,w[i],dat.time-times[i]
    
endfor
energy = average(dat.energy,2)

if not keyword_set(postfix) then postfix = strtrim(npad,2)

newname = dname+'pd'+postfix
store_data,newname,data={x:times,y:data,v1:energy,v2:pab},dlim={comment:'units: '+units}

reduce_pads,newname,2,0,0
reduce_pads,newname,2,np-1,np-1
reduce_pads,newname,2,np/2-1,np/2
nn = dat.nenergy
; for i=0,nn-1 do reduce_pads,newname,1,i,i

end
