
pro get_elpd,trange=trange,data=d,cdf_time=cdf_time $
   ,nsource=nsource,test=test,skip=skip,no_catch=no_catch  $
   ,no_reduce=no_reduce,do_fit=do_fit,verbose=verbose,timebar=tbar


do_fit=keyword_set(do_fit)

if keyword_set(cdf_time) then begin
    suffix = do_fit ? '_v06' : '_v05'
    if keyword_set(skip) then suffix = '_v04'
    datelab = time_string(cdf_time[0],format=2,/date_only)+suffix
    fname = full_pathname('wi_3dp_elpd_'+datelab,/mkdir)
    if file_test(/regular,fname+'.cdf') then return
    trange = cdf_time
endif

load_3dp_data,/if_needed

ex_start = systime(1)

units='flux'
bsource = 'wi_B3'
if not keyword_set(vsource) then vsource = 'Vp'
if not keyword_set(nsource) then nsource = 'Np'
if not keyword_set(tsource) then tsource = 'Tp'
if not keyword_set(possource) then possource = 'wi_pos'

data_str = 'el'
routine = 'get_'+data_str
times0 = call_function(routine,/times)
if ndimen(times0) le 0 then begin
   message,/info,'No electron data!'
   return
endif

load_if_needed,bsource
load_if_needed,nsource
load_if_needed,possource

if keyword_set(num_pa) eq 0 then num_pa = 13



max = n_elements(times0)
if keyword_set(trange) then begin
;  timebar,trange
  tr = time_double(trange)
  ind = where(times0 le tr[1] and times0 ge tr[0],max)
endif else ind = lindgen(max)

if keyword_set(skip) then begin
  if skip eq 1 then skip2 = (max/50) > 1
  s = where((lindgen(max)  mod skip2) eq 0)
  ind = ind[s]
endif

times = times0[ind]
max = n_elements(times)

nenergy = 15
;nredf = 32

nan = !values.f_nan
m0=mom3d()

fc0 =  fitel3d(fitresult=fitresult)   ; initial guess
;fc0 = sum2elfit(mom_sum())     ; initial guess

d0={time:!values.d_nan,valid:0,nsw:nan,vsw:replicate(nan,3),tsw:nan $
  ,flags:0l $
  ,magf:replicate(nan,3) $
  ,pos: replicate(nan,3) $
  ,sc_pot_nsw: nan $
  ,sc_pot_el: nan $
  ,mom:m0  $
  ,omni_nrg: replicate(nan,nenergy) ,omni_eflux: replicate(nan,nenergy), omni_stdev: replicate(nan,nenergy)  $  
  ,flux:replicate(nan,nenergy,num_pa) $
  ,energy:replicate(nan,nenergy),pangle:replicate(nan,num_pa) } 
  
if do_fit then begin 
  str_element,/add,d0,'fit',fill_nan(fc0)
  str_element,/add,d0,'fitres',fitresult
endif
 
if keyword_set(d) then old_d =d
d = replicate(d0,max)
d.time = times
;if n_elements(old_d) eq n_elements(d) then begin
;stop
;   d = old_d 
;endif


;interp_gap = 1
;gap_dist = 35
gap_thresh = 300.  ; seconds
d.pos = transpose(data_cut(possource,times))
d.vsw = transpose(data_cut(vsource,times,gap_thresh=gap_thresh))
d.magf= transpose(data_cut(bsource,times,gap_thresh=gap_thresh))
d.nsw = data_cut(nsource,times,gap_thresh=gap_thresh)
d.tsw = data_cut(tsource,times,gap_thresh=gap_thresh)

ethresh = 300.



for i=0,max-1 do begin
   
   if keyword_set(do_fit) then begin
     wait_for_others
     if  not keyword_set(no_catch) then begin
       catch,error_status
       if error_status ne 0 then begin
   ;      crlf = string(13b)+string(12b)
         message = 'DATA Time: '+ time_string(times[i])
         message = [message, !error_state.msg]
         page,message ,subject='Error in get_elpd'
         continue;
       endif  
     endif
   endif
   
   dat = call_function(routine,index=ind[i])

   if keyword_set(verbose) then $
     print,string(13b),i,max,ind[i],time_string(dat.time),format='(%"%s%d of %d  (%d)  %s   ",$)'
   d[i].time = dat.time
   
   true_dens = d[i].nsw
   if finite(true_dens) eq 0 or true_dens lt .1 then continue
   rpos = sqrt(total(d[i].pos^2))
   if rpos lt 15 then continue
   
   d[i].valid= dat.valid
   if dat.valid eq 0 then begin
   	print,i,' ',time_string(dat.time), ' Invalid data'
   	continue
   endif
     if keyword_set(tbar) then   timebar,dat.time,/transient
     if times[i] ne dat.time then print,time_string(dat.time),dat.time-times[i]
     dat.magf = d[i].magf
     dat.vsw =  d[i].vsw
     d[i].sc_pot_nsw = sc_pot(d[i].nsw)
     d[i].sc_pot_el = sc_pot_el(dat)
     pot = d[i].sc_pot_nsw
     pot = 1. > pot < 40.
     if keyword_set(debug) then stop
     if do_fit then begin
        true_dens = d[i].nsw
;        if finite(d[i].fit.core.n)  then fc = d[i].fit
        m = mom_el(dat,true_dens=true_dens,format=m0,fit_core=fc,test=test,fitresult=fitresult)
        if keyword_set(fitresult) then d[i].fitres = fitresult
        if keyword_set(fc) then d[i].fit = fc
     endif else begin
        m =  mom3d(dat,sc_pot=pot,format=m0)
    endelse
     omni = conv_units(dat,'flux')
     d[i].omni_nrg = average(omni.energy,2)
     d[i].omni_eflux = average(omni.data,2,stdev=stdev)
     d[i].omni_stdev = stdev
     d[i].mom = m
     dat.sc_pot = m.sc_pot
     df = convert_vframe(dat,d[i].vsw,/interp,ethresh=ethresh)
     df = conv_units(df,units)
     pd = pad(df,magf=dat.magf,NUM_PA=num_pa,BINS=bins)
     d[i].flux = pd.data
     d[i].energy = pd.energy[*,num_pa/2]
     d[i].pangle = pd.angles[nenergy/2,*]
     if keyword_set(tbar) then   timebar,dat.time,/transient
     if keyword_set(break) then break
  
endfor
catch,/cancel

if keyword_set(debug) then stop

elpd_str_to_tplot,d,do_fit=do_fit ;,no_reduce=no_reduce

if keyword_set(cdf_time) then begin
    w=where(d.time ge cdf_time[0] and d.time lt cdf_time[1],n)
    if n ge 0 then    makecdf,d[w],file=   fname,/over  $
    else message,'No data in cdf_time range!' 
    if  keyword_set(do_fit) then save,file=fname+'.sav',d
    if keyword_set(skip) then name='skip_'+datelab else name='full_'+datelab
    name = '/home/davin/dat/wi/3dp/elpd/v06/'+name
    mk_elpd_sumplot2,plot=0,name=name+'_a'
    mk_elpd_sumplot2,plot=1,name=name+'_b'
endif

end

