pro magnetogram, file,limits=lim,tshift=tshift,noplot=noplot,xt=xt,yt=yt,zt=z $
  ,time=tmiddle

common magnetogram_com, path

if not keyword_set(path) then path = '~davin/dat/solar/kittpeak'


if data_type(file) ne 7 then $
  file=dialog_pickfile(path=path,get_path=path,/read,filter='??????m')

if not keyword_set(file) then return


if data_type(lim) ne 8 then begin
  lim=0
  zlim,lim,-50,50
  xlim,lim,-1.05,1.05
  ylim,lim,-1.05,1.05
  options,lim,'x_no_interp',1
  options,lim,'y_no_interp',1
endif


image = readfits(file,h,/nosc)
bscale = sxpar(h,'BSCALE')
if bscale eq 0 then bscale = 1.
bzero = sxpar(h,'BZERO')
t1 = double(sxpar(h,'STARTIME'))
t2 = double(sxpar(h,'ENDTIME'))
nx = dimen1(image)
ny = dimen2(image)

tmiddle = (t1+t2)/2

p={x0:0.,y0:0.,r0:0.}
p.x0 = sxpar(h,'E_XCEN')/4.
p.y0 = sxpar(h,'E_YCEN')/4.
p.r0 = 3600./ (sxpar(h,'scale')*4.) *.27

x = (findgen(nx)-p.x0)/p.r0
y = (findgen(ny)-p.y0)/p.r0
xt = x # replicate(1.,ny)
yt = replicate(1.,ny) # y
rt = sqrt(xt^2 + yt^2)
z = (image+.5) * bscale + bzero
bad = where(rt gt .99 and z lt -200,count)
if count ne 0 then z(bad) = !values.f_nan
print,time_string(t1),bscale,min(z,/nan),max(z,/nan)
options,lim,'title','Kitt Peak Magnetogram  '+time_string(t1)

if keyword_set(tshift) then begin
   r = 1.
   w = where(rt lt .999)
   theta = asin(yt(w)/r)
   phi = asin(xt(w)/r/cos(theta))
   dphi = tshift/(28.*24.*3600.)*2 *!pi
   phi = phi+dphi
   xt(w) = r*sin(phi)*cos(theta) 
   tmiddle=tmiddle-tshift
endif


options,lim,'position',plot_pos(lim=lim,/top)
if not keyword_set(noplot) then  specplot,x,y,z,lim=lim
levels = (findgen(10)-4.5)*200
;box,lim & contour,z,xt,yt,lev=levels,over,/fill
;contour,z,x,y,lev=levels,/over,col=0

;a = findgen(360)/360.*2*!pi
;oplot,sin(a),cos(a)


end
