function min_curv,vx,vy,vz, data=df,parameter=dfpar, log=log $
   ,linear=linear,quadratic=quadratic,y_symmetric=y_symmetric, $
   , p_names = p_names, pder_values= pder_values

dim = n_params()
;if dim eq 1 then begin
;    return, min_curv(vx.x,vx.y,data=df,parameter=dfpar,log=log)
;endif

if keyword_set(df) then begin
   logflag = keyword_set(log)
   linbasis = keyword_set(linear)
   quadbasis = keyword_set(quadratic)
   y_symmbasis = keyword_set(y_symmetric)
   if logflag then df0 = alog(df) else df0=df

   good = where(finite(vx) and finite(vy) and finite(df0))   ; valid points only
   vx0 = double(vx[good])
   vy0 = double(vy[good])
   if dim eq 3 then  vz0 = double(vz[good])
   df0 = double(df0[good])

   n = n_elements(df0)

   m = n +  1   ;  m = # of eqns to solve
   if linbasis  then m = m + dim - y_symmbasis
   if quadbasis then m = m + dim
   a = dblarr(m, m)     ;LHS

   for i=0, n-1 do for j=i+1,n-1 do begin
      d1 = ((vx0[i]-vx0[j])^2 + (vy0[i]-vy0[j])^2)
      if dim eq 3 then d1 = d1 + (vz0[i]-vz0[j])^2
      d = d1*alog(d1 eq 0 ? 1 : d1)
      if y_symmbasis then begin
         d1 = ((vx0[i]-vx0[j])^2 + (vy0[i]+vy0[j])^2)
         d += d1*alog(d1 eq 0 ? 1 :d1)
      endif
;d=d1
      a[j,i] = d
   endfor

   i = n
   a[i++,0:n-1] = 1.   ;   constant   fill rest of array
   if linbasis then begin
     a[i++,0:n-1] = vx0    ; linear terms
     if ~y_symmbasis then a[i++,0:n-1] = vy0
     if dim eq 3 then begin
        a[i++,0:n-1] = vz0
     endif
   endif
   if quadbasis then begin
     a[i++,0:n-1] = vx0^2   ; quadratic terms
     a[i++,0:n-1] = vy0^2
     if dim eq 3 then begin
        a[i++,0:n-1] = vz0^2
     endif
   endif

   if i ne m then message, 'Programming error.'

   ;fill lower triangle
   for i=0, m-1 do for j=i,m-1 do begin
     a[i,j] = a[j,i]
   endfor

   b = dblarr(m)
   b[0:n-1] = df0

   c = reform(b # invert(a))

   dfpar = {dim:dim,logflag:logflag,linbasis:linbasis,quadbasis:quadbasis,y_symmbasis:y_symmbasis,dfc:c,x0:vx0,y0:vy0}
   if dim eq 3 then dfpar = create_struct(dfpar,'z0',vz0)
   return,dfpar
endif

dim = dfpar.dim
npnames = n_elements(p_names)
;stdflag = dfpar.stand

vx0=dfpar.x0
vy0=dfpar.y0

if dim eq 3 then vz0=dfpar.z0
c = dfpar.dfc

n = n_elements(vx0)
s = 0
for i=0, n-1 do begin
   d2 = ((vx0[i]-vx)^2 + (vy0[i]-vy)^2)
   if dim eq 3 then d2 = d2 + (vz0[i]-vz)^2
   d = d2*alog(d2 > 1d-100)
   if dfpar.y_symmbasis then begin
      d2 = ((vx0[i]-vx)^2 + (vy0[i]+vy)^2)
      d += d2*alog(d2 > 1d-100)
   endif
;d=d1
   s = s + d * c[i]
;   for j=0,npnames-1 do begin
;   endfor
endfor

i = n
s = s + c[i++]

if dfpar.linbasis then begin
   s=  s+  c[i++] * vx
   if ~dfpar.y_symmbasis then s =  s +  c[i++] * vy
   if dim eq 3 then begin
      s = s+ c[i++] * vz
   endif
endif
if dfpar.quadbasis then begin
   s=  s+  c[i++] * vx^2
   s=  s+  c[i++] * vy^2
   if dim eq 3 then begin
      s = s+ c[i++] * vz^2
   endif
endif




if i ne n_elements(c) then message,'Programming error'


if keyword_set(debug) then stop

if dfpar.logflag then s = exp(s)
return,s


end




;Test code:

xlim,lim,0,1
ylim,lim,0,1
zlim,lim,-1,1
options,lim,xmargin=[10,10]

xv=dgen(range=lim.xrange)
yv=dgen(range=lim.yrange)

xy= xv # replicate(1,n_elements(yv))
yx= replicate(1,n_elements(xv)) # yv

fv_ex = sin(xy*5*!pi)*sin(yx*!pi*2)
specplot,lim=lim,xv,yv,fv_ex
contour,level=[-.5,0,.5],fv_ex,xv,yv,/over

if keyword_set(npoints) then  begin
  x = randomu(seed,npoints)
  y = randomu(seed,npoints)
endif else getxy,x,y

f = sin(x*5*!pi)*sin(y*!pi*2)             ; get data

par = min_curv(x,y,data=f)                ; get parameters
printdat,par

fv = min_curv(xy,yx,param=par)            ; calculate interpolated values


levels = [-1,-.5,0,.5,1]
specplot,lim=lim,xv,yv,fv
contour,level=levels,fv,xv,yv,/over
oplot,x,y,/ps
contour,level=levels,fv_ex,xv,yv,/over
stop

lim.zrange /= 10
specplot,lim=lim,xv,yv,fv-fv_ex
contour,level=levels/10,fv-fv_ex,xv,yv,/over
oplot,x,y,/ps


end
