;idl

function orb_body,sun=sun,earth=earth,moon=moon
body = { orb_body,   $
         name:          '',    $
         number:       0l ,    $
         G_mass:       0.d,    $
;         mass:         0.d,    $    ; kg  (for drag purposes only)
         radius:       0.d,    $
         rad_pol:      0.d,    $    ; polar radius
         rad_equ:      0.d,    $    ; equatorial radius
         oblateness:   0.d,    $    ; G*(I3-I1)
         spin:   dblarr(3),    $    ; spin vector
         time:         0.d,    $
         x:      dblarr(3),    $
         v:      dblarr(3),    $
         relname:       '',    $
         rel:          -1l,    $
         GM_center:    0.d,    $
         orbit:         0l,    $
         rx:     dblarr(3),    $
         rv:     dblarr(3),    $
         r:            0.d,    $   ; separation distance
         phi:          0.d,    $   ; (true anomaly)
         m_anomaly:    0.d,    $   ; mean anomaly
         e_anomaly:    0.d,    $   ; eccentric anomaly
         apapsis:      0.d,    $   ;
         periapsis:    0.d,    $
         semimajor:    0.d,    $
         t_per0:       0.d,    $
         m_motion:     0.d,    $   ; 360 / period
         top:          0.d,    $   ;
; Orbital elements:
         period:       0.d,    $
         eccent:       0.d,    $
         inclin:       0.d,    $
         raan:         0.d,    $
         argp:         0.d,    $
         t_per:        0.d     }
return,body
end


function jday,time
return, time_double(time)/86400.d + 2440587.5d
end



function sun_earth_moon,date=date,dir=dir
   if not keyword_set(dir) then begin
     scp = scope_traceback(/struc)
     n = n_elements(scp)
     dir  = file_dirname(scp[n-1].filename)+'/'
   endif
   if n_elements(date) eq 0 then date=systime(1)
   re = 6374.d
   body = orb_body()
   jd = jday(date)
   sun = body
   sun.name = 'Sun'
   sun.relname = 'Earth'
   emb = read_orb_elem(dir+'emba.ele',time=jd ,format=body)
   sun.g_mass = emb.gm_center / re^3
   set_state,emb,sun

   moon = read_orb_elem(dir+'moon.ele',time=jd ,format=body)


   earth = body
   earth.name = 'Earth'
   earth.relname = 'Sun'

   mm_me = 0.012300034d
   gm = moon.gm_center / re^3
   earth.G_mass = gm / (1+mm_me)
   moon.G_mass = gm * mm_me / (1+mm_me)

   set_state,moon,earth
   earth.x = emb.x + moon.rx *  moon.G_mass/(earth.G_mass+moon.G_mass)
   moon.x  = emb.x - moon.rx * earth.G_mass/(earth.G_mass+moon.G_mass)
   earth.v = emb.v + moon.rv *  moon.G_mass/(earth.G_mass+moon.G_mass)
   moon.v  = emb.v - moon.rv * earth.G_mass/(earth.G_mass+moon.G_mass)


   earth.radius = 1.d
;   earth.G_mass = 398600.440d ; km^3 s^-2
   earth.oblateness = 0.00327 * .3308 * earth.G_mass * earth.radius^2
   earth.oblateness = 0.0010826265 * earth.G_mass * earth.radius^2
   sem = [sun,earth,moon]
   sem.time=jd
   set_orbital_elements,sem
   return,sem
end


pro set_relative,bods
   bods.number = indgen(n_elements(bods))
   for i=0,n_elements(bods)-1 do begin
      rel = where(bods.name eq bods[i].relname,c)
      if c ne 0 then bods[i].rel = rel[0]
   endfor
end








pro init_magcat

common orbit_com,  all

   start = '2004-02-05'
;   AU = 1.49597870700d11   ; m
;   G = 6.67259d-11; meters^3 kilograms^-1 seconds^-2
;   GM_sun = 1.32712442007d20 ;  m^3 s^-2

   body = orb_body()

   sem = sun_earth_moon(date=start)

   sun   = sem[0]
   earth = sem[1]
   moon  = sem[2]

   apo1 = 13.75d
   apo2 = 9.75d
   apo1 = 15d
   apo2 = 10d
   per1 = 1.025d
   per1 = 1.079d
   per2 = 1.079d
   per1 = 1.1d
   per2 = 1.1d

   for i=0,10 do $
        per2 = apo1*per1/apo2 * ((apo2+per2)/(apo1+per1)) ^ .25d

   nsats1 = 10
   nsats2 = 6
   nsats1 = 1
   nsats2 = 1
;   nsats1 = 0
;   nsats2 = 0
   nsats = nsats1+nsats2
   if nsats ge 1 then begin
    st1 = indgen(nsats1)
    st2 = indgen(nsats2) + nsats1
    sats = replicate(body,nsats)
    sats.name='s'+strtrim(sindgen(nsats),2)
    sats.relname='Earth'
    sats.g_mass=0.d
    sats[st1].periapsis = per1
    sats[st2].periapsis = per2
    sats[st1].apapsis   = apo1
    sats[st2].apapsis   = apo2
    set_period_ecc,sats,(earth.G_mass+sats.G_mass)
    sats[st1].inclin = 90d
    sats[st2].inclin = 90d
    sats[st1].inclin = 28.5d
    sats[st2].inclin = 28.5d
    sats.raan        =  90.d  ; +moon.raan
    radeg = 180.d/!dpi
    x =  (earth.oblateness/earth.G_mass) / sats.semimajor^2 / (1-sats.eccent^2)^2 / sats.period
    argp_dot =  .75 * 360. * x * (5*cos(sats.inclin/radeg)^2-1)
    raan_dot = -1.5 * 360. * x * cos(sats.inclin/radeg)
    print,'argp_dot',argp_dot * 365.25d
    print,'raan_dot',raan_dot * 365.25d
;    sats.argp        = 180.d + 23.45d + 9d
    sats.argp        = 90d
    if nsats1 gt 1 then sats[st1].t_per  = sats[st1].period *(.5+findgen(nsats1))/nsats1
    if nsats2 gt 1 then sats[st2].t_per  = sats[st2].period *(.5+findgen(nsats2))/nsats2

    set_state,sats,replicate(earth,nsats)

    bus = sats[0]
    bus.apapsis = 6.5d
    bus.periapsis = 1.029
    set_period_ecc,bus,(earth.G_mass+bus.G_mass)
    bus.name = 'bus'
    set_state,bus,earth

    all = [sun,earth,moon,sats,bus]
   endif else     all = [sun,earth,moon]
   all.number = indgen(n_elements(all))

   cent_mass = body
   mass3 = [1,1,1] # all.g_mass
   cent_mass.x = total(all.x * mass3,2) / total(mass3,2)
   cent_mass.v = total(all.v * mass3,2) / total(mass3,2)
   cent_mass.g_mass = 0.d

   all.time=0
   all.orbit = 0
   all.t_per = 0
   set_orbital_elements,all
end






pro init_blast

common orbit_com,  all

   start = '2004-02-05'
;   AU = 1.49597870700d11   ; m
;   G = 6.67259d-11; meters^3 kilograms^-1 seconds^-2
;   GM_sun = 1.32712442007d20 ;  m^3 s^-2

   body = orb_body()

   sem = sun_earth_moon(date=start)

   sun   = sem[0]
   earth = sem[1]
   moon  = sem[2]

   apo1 = 15d
   apo2 = 15d
   per1 = 1.4d
   per2 = 1.4d
   per2 = 1.4001d

;   for i=0,10 do $
;        per2 = apo1*per1/apo2 * ((apo2+per2)/(apo1+per1)) ^ .25d

   nsats = 1
   if nsats ge 1 then begin
    sats = replicate(body,nsats)
    sats.name='s'+strtrim(sindgen(nsats),2)
    sats.relname='Earth'
    sats.g_mass=0.d
    sats.periapsis = 1.3
    sats.apapsis   = 10
    sats.inclin = 90d
    sats.argp   = 27d
    if nsats gt 1 then sats.argp = sats.argp + dindgen(nsats)/nsats*360
    sats.raan   =  0.d
    radeg = 180.d/!dpi
    set_period_ecc,sats,(earth.G_mass+sats.G_mass)
    x =  (earth.oblateness/earth.G_mass) / sats.semimajor^2 / (1-sats.eccent^2)^2 / sats.period
    argp_dot =  .75 * 360. * x * (5*cos(sats.inclin/radeg)^2-1)
    raan_dot = -1.5 * 360. * x * cos(sats.inclin/radeg)
    print,'argp_dot',argp_dot * 365.25d
    print,'raan_dot',raan_dot * 365.25d

    set_state,sats,replicate(earth,nsats)


    all = [sun,earth,moon,sats]
   endif else     all = [sun,earth,moon]
   all.number = indgen(n_elements(all))

   cent_mass = body
   mass3 = [1,1,1] # all.g_mass
   cent_mass.x = total(all.x * mass3,2) / total(mass3,2)
   cent_mass.v = total(all.v * mass3,2) / total(mass3,2)
   cent_mass.g_mass = 0.d

   all.time=0
   all.orbit = 0
   all.t_per = 0
   set_orbital_elements,all
end


pro init_themis, start

common orbit_com,  all

   if not keyword_set(start) then start = systime(1)-24d*3600d*2
;   AU = 1.49597870700d11   ; m
;   G = 6.67259d-11; meters^3 kilograms^-1 seconds^-2
;   GM_sun = 1.32712442007d20 ;  m^3 s^-2
   re = 6374.d

   body = orb_body()

   sem = sun_earth_moon(date=start)

   sun   = sem[0]
   earth = sem[1]
   moon  = sem[2]

;   apo1 = 15d
;   apo2 = 15d
;   per1 = 1.4d
;   per2 = 1.4d
;   per2 = 1.4001d

;   for i=0,10 do $
;        per2 = apo1*per1/apo2 * ((apo2+per2)/(apo1+per1)) ^ .25d

   nsats = 5
   if nsats ge 1 then begin
    sats = replicate(body,nsats)
    sats.name='s'+strtrim(sindgen(nsats),2)
    sats.relname='Earth'
    sats.g_mass=0.d
    probes = 'th'+['a','b','c','d','e']

    for i=0,nsats-1 do begin
       sats[i].rx = data_cut(probes[i]+'_state_pos',start) / re
       sats[i].rv = data_cut(probes[i]+'_state_vel',start) / re * (24d*3600.d)
       sats[i].x =  earth.x + sats[i].rx
       sats[i].v =  earth.v + sats[i].rv
    endfor

    set_orbital_elements,sats,primary=replicate(earth,nsats)

    radeg = 180.d/!dpi
;    set_period_ecc,sats,(earth.G_mass+sats.G_mass)
    x =  (earth.oblateness/earth.G_mass) / sats.semimajor^2 / (1-sats.eccent^2)^2 / sats.period
    argp_dot =  .75 * 360. * x * (5*cos(sats.inclin/radeg)^2-1)
    raan_dot = -1.5 * 360. * x * cos(sats.inclin/radeg)
    print,'argp_dot',argp_dot * 365.25d
    print,'raan_dot',raan_dot * 365.25d

;    set_state,sats,replicate(earth,nsats)

    all = [sun,earth,moon,sats]
   endif else     all = [sun,earth,moon]
   all.number = indgen(n_elements(all))

   cent_mass = body
   mass3 = [1,1,1] # all.g_mass
   cent_mass.x = total(all.x * mass3,2) / total(mass3,2)
   cent_mass.v = total(all.v * mass3,2) / total(mass3,2)
   cent_mass.g_mass = 0.d

   all.time=0
   all.orbit = 0
   all.t_per = 0
   set_orbital_elements,all
end


pro set_period_ecc,s,Gm
s.semimajor = (s.periapsis+s.apapsis)/2
s.eccent = (s.apapsis-s.periapsis)/2/s.semimajor
s.period = 2.d * !dpi / sqrt( Gm / s.semimajor^3.d)
end





pro set_state,s,p

   if not keyword_set(p) then begin
     for i=0,n_elements(s)-1 do begin
       rel = where(s.name eq s[i].relname,c)
       if c ne 0 then s[i].rel = rel[0]
     endfor
     p = s[s.rel]
   endif
   Gm =  (s.G_mass + p.G_mass)

   s.semimajor = ((s.period/2/!dpi)^2*Gm)^(1/3.d)
   s.m_anomaly = (s.time - s.t_per)/s.period * 2.d *!dpi
   s.e_anomaly = e_anomaly(s.m_anomaly,s.eccent)

   tan_ea_2 = tan(s.e_anomaly/2.d)
   ee = (1+s.eccent)/(1-s.eccent)

   s.r = s.semimajor*(1-s.eccent*cos(s.e_anomaly))
   s.phi  = 2. * atan( sqrt(ee) * tan_ea_2 )         ; true_anomaly

   radeg = 180.d/!dpi
   inclin = s.inclin / radeg
   raan   = s.raan / radeg
   argp   = s.argp / radeg

   u = s.phi + argp
   sin_u = sin(u)
   cos_u = cos(u)
   sin_i = sin(inclin)
   cos_i = cos(inclin)
   sin_raan = sin(raan)
   cos_raan = cos(raan)

   s1  =  ( cos_u * cos_raan - sin_u * sin_raan * cos_i )
   s2  =  ( cos_u * sin_raan + sin_u * cos_raan * cos_i )
   s3  =  sin_u * sin_i

   x  = s.r * s1
   y  = s.r * s2
   z  = s.r * s3

   s.rx[0] = x
   s.rx[1] = y
   s.rx[2] = z

   ea_dot = 2.d*!dpi/s.period / (1-s.eccent*cos(s.e_anomaly))
   r_dot = s.semimajor * s.eccent * sin(s.e_anomaly) * ea_dot
   phi_dot = sqrt(ee) * (1+tan_ea_2 ^2)/(1+ee*tan_ea_2 ^2) * ea_dot

;   speed = sqrt(r_dot^2 + (s.r * phi_dot)^2)

   x_dot = r_dot * s1 + s.r * (-sin_u * cos_raan - cos_u * sin_raan * cos_i) * phi_dot
   y_dot = r_dot * s2 + s.r * (-sin_u * sin_raan + cos_u * cos_raan * cos_i) * phi_dot
   z_dot = r_dot * s3 + s.r * ( cos_u * sin_i) * phi_dot

   s.rv(0) = x_dot
   s.rv(1) = y_dot
   s.rv(2) = z_dot

   if n_elements(s) eq n_elements(p) then begin
     s.x = s.rx + p.x
     s.v = s.rv + p.v
   endif else begin
     s.x = s.rx + p.x # replicate(1.d,n_elements(s))
     s.v = s.rv + p.v # replicate(1.d,n_elements(s))
   endelse

end




; warning! this code may not properly set raan and argp if inclination is ge 90.
;   ignore above warning.

; Error!  code is only accurate if mass of satellite is much less than mass
;   of planetary body.  I have not yet found the error. maybe!

pro  set_orbital_elements,s,time0,pos,vel,primary = primary

if keyword_set(time0) then s.time = time0
if keyword_set(pos)  then s.x = pos
if keyword_set(vel)  then s.v = vel

if not keyword_set(primary) then begin
   for i=0,n_elements(s)-1 do begin
      rel = where(s.name eq s[i].relname,c)
      if c ne 0 then s[i].rel = rel[0]
   endfor
   primary = s[s.rel]
endif

if n_elements(s) eq n_elements(primary) then begin
  s.rx = s.x - primary.x
  s.rv = s.v - primary.v
endif else begin
  s.rx = s.x - primary.x # replicate(1.d,n_elements(s))
  s.rv = s.v - primary.v # replicate(1.d,n_elements(s))
endelse

x=  s.rx(0)
y=  s.rx(1)
z=  s.rx(2)
vx =  s.rv(0)
vy =  s.rv(1)
vz =  s.rv(2)

time = s.time
Gm = primary.G_mass +s.G_mass

a1 = x*vy - y*vx
a2 = y*vz - z*vy
a3 = z*vx - x*vz
c12 = a1^2 + a2^2 +a3^2
c1 = sqrt(c12)

incl = acos(a1/c1)
;sign = (a1 ge 0)*2 -1
;raan = atan(sign*a2,-sign*a3)
raan = atan(a2,-a3)
p = c12/Gm   ;semiparameter
r2 = x^2+y^2+z^2
r = sqrt(r2)
v2 = vx^2 + vy^2 + vz^2
c3 = v2 - 2.d*Gm/r
ecc2 = 1+c12*c3/Gm^2
a = p/(1-ecc2)  ;semimajor axis
rdot_r = x*vx+y*vy+z*vz          ; rdot x r
T = 2.d * !dpi / sqrt( Gm / a^3.d)     ;Period
ea = atan(rdot_r*T/a^2/2.d/!dpi, 1-r/a)    ;eccentric anomaly
ecc = sqrt(ecc2)         ;eccentricity
ma = ea - ecc*sin(ea)     ;mean anomaly
phase = ma/2/!dpi
phase = phase - (phase ge 0) ; force phase negative

t_per = time - phase * T   ;time of next perigee
ta = 2.d*atan( sqrt((1+ecc)/(1-ecc)) *tan(ea/2))  ; true anomaly

; finally; argument of perigee:
xp = cos(raan) * x          + sin(raan)*y
yp = -cos(incl)*sin(raan)*x + cos(incl)*cos(raan)*y + z*sin(incl)
u = atan(yp,xp)
argp = u - ta
argp = argp + 2.d*!dpi * (argp lt 0.)

radeg = 180.d/!dpi

s.r  = r
s.phi = ta

s.semimajor = a
s.eccent = ecc
s.period = T
s.e_anomaly = ea
s.m_anomaly = ma
s.inclin   = incl * radeg
s.raan     = raan * radeg
s.argp     = argp * radeg
s.periapsis = s.semimajor * (1-s.eccent)
s.apapsis = s.semimajor * (1+s.eccent)

inc = round((t_per - s.t_per)/s.period)

s.orbit = s.orbit + inc
s.t_per = t_per
s.t_per0 = s.t_per - s.orbit * s.period
s.top = s.t_per - s.time
end





pro precession_rates,s
p = s[s.rel]
radeg = 180.d/!dpi
x =  (p.oblateness/p.G_mass) / s.semimajor^2 / (1-s.eccent^2)^2 / s.period
raan_dot = -1.5 * 2 * !dpi * x * cos(s.inclin/radeg)
argp_dot =  .75 * 2 * !dpi * x * (5*cos(s.inclin/radeg)^2-1)
; these are from goldstein page 514. Note that formula for argp_dot has been changed!
print,"argp:"
print,argp_dot*radeg
print
print,"raan:"
print,raan_dot*radeg
end



function massdensity,a
common density_com, alt,o2,o,n2,mdens
if not keyword_set(mdens) then begin
  mp = 1.67e-27  ; kg
  restore,file='~davin/cat/density.dat',/verbose
  mdens = alog10((32.*10^o2 + 16*10^o + 28*10^n2)*mp*1d15)
endif
return,10^interp(mdens,alt,a)   ; mass density is kg/km^3
end




function altitude,dx,dy,dz,r_e,r_p
dr2 = dx^2 + dy^2 + dz^2
cos2_th = dz^2/dr2
r = r_p + (r_e-r_p)*cos2_th
return,sqrt(dr2) - r
end




function orbderiv,t,vec

common orbit_com,  all

;Note calculated forces are really force per unit mass.

nsat = n_elements(all)

tvec = reform(vec,3,2,nsat)
;s.x = tvec[*,0,*]
;s.v = tvec[*,1,*]

diag = lindgen(nsat)*(nsat+1)

i = replicate(1.d,nsat)
x = reform(tvec[0,0,*])
y = reform(tvec[1,0,*])
z = reform(tvec[2,0,*])
dx = (i # x) - (x # i)
dy = (i # y) - (y # i)
dz = (i # z) - (z # i)
d2 = dx^2 + dy^2 + dz^2
d2(diag) = 1.d              ; Note diagonals forced to be non-zero to avoid 0 divide by 0
d3 = d2^1.5d

;  Gravitational forces:

Gm = all.G_mass # i

fx = - total(Gm * dx / d3, 1)
fy = - total(Gm * dy / d3, 1)
fz = - total(Gm * dz / d3, 1)

; include oblateness of the Earth
;  This code is only valid if the principle axis of all bodies are
;  aligned with the coordinate system.  (i.e. spin vector is ignored)
; I am unsure about these equations they should be checked.  - sign added feb 98

if 1 then begin
obl = all.oblateness # i
dz2_dr2 = dz^2/d2
d5 = d2^2.5d
fx = fx -  total(obl * (3.d - 15.d* dz2_dr2) * dx / d5 ,1) /2.d
fy = fy -  total(obl * (3.d - 15.d* dz2_dr2) * dy / d5 ,1) /2.d
fz = fz -  total(obl * (9.d - 15.d* dz2_dr2) * dz / d5 ,1) /2.d
endif else begin
  shatx = all.spin[0] # i
  shaty = all.spin[1] # i
  shatz = all.spin[2] # i
  sdr = shatx*dx + shaty*dy   + shatz*dz
  foo = (3.d - 15.d*sdr/d2)
  fx = fx -  total(obl * (foo*dx + 6*sdr*shatx) / d5 ,1) /2.d
  fy = fy -  total(obl * (foo*dy + 6*sdr*shaty) / d5 ,1) /2.d
  fz = fz -  total(obl * (foo*dz + 6*sdr*shatz) / d5 ,1) /2.d

;ind = where(all.oblateness ne 0,n)
;for j=0,n-1 do begin
;   b = all[ind[j]]
;   spinrate = sqrt(total(b.spin^2))
;   if spinrate eq 0 then shat =[0,0,1d] else shat=b.spin/spinrate
;   fx = fx - b.oblateness * (
;   message,'Error, not finished'
;endfor
endelse

;  Include aerodynamic drag:

if 0 then begin
  vx = reform(tvec[0,1,*])
  vy = reform(tvec[1,1,*])
  vz = reform(tvec[2,1,*])
  dvx = (i # vx) - (vx # i)
  dvy = (i # vy) - (vy # i)
  dvz = (i # vz) - (vz # i)
  dv  = sqrt(dvx^2 + dvy^2 + dvz^2)
  alt = altitude

endif


;  Include thrusters:


; Set state vector:

vecdot = dblarr(3,2,nsat)
vecdot[*,0,*] = tvec[*,1,*]
vecdot[*,1,*] = reform(transpose([[fx],[fy],[fz]]),3,1,nsat)

return,reform(vecdot,6*nsat)
end



function rk4_new,vec,dvecdt,t,dt,fname,double=double
   vec0 = vec

   dvecdt = call_function(fname,t,vec)
   dvec = dvecdt
   vec = vec0 + dt * dvecdt/2

   dvecdt = call_function(fname,t+dt/2.d,vec)
   dvec = dvec + 2.d * dvecdt
   vec = vec0 + dt * dvecdt/2

   dvecdt = call_function(fname,t+dt/2,vec)
   dvec = dvec + 2.d * dvecdt
   vec = vec0 + dt * dvecdt

   dvecdt = call_function(fname,t+dt,vec)
   dvec = dvec + dvecdt
   vec = vec0 + dt*dvec/6
   return,vec
end




pro plot_struct,str,names=names

if not keyword_set(names) then names=tag_names(str)
if ndimen(names) eq 0 then nm = strsplit(names,/extract) else nm=names
pos = array_union(strupcase(nm),tag_names(str))
nt = n_elements(nm)
!p.multi=[0,0,nt]
!y.margin = 1
options,lim,'ynozero',1
options,lim,'charsize',2.
for b=0,nt-1 do begin
  options,lim,'ytitle',nm[b]
  mplot,transpose(str.time),transpose(str.(pos[b])),lim=lim
endfor
!p.multi=0
end


pro plot_struct2,str,names=names

if not keyword_set(names) then names=tagnames(str)
if ndimen(names) eq 0 then nm = strsplit(names,/extract) else nm=names
pos = array_union(strupcase(nm),tag_names(str))
nt = n_elements(nm)
!p.multi=[0,0,nt]
options,lim,'ynozero',1

wall = where(str.number ge 3)

for b=0,nt-1 do begin
  plot,str[wall].time,str[wall].(pos[b]),ytitle=nm[b],/nodata,/ynoz,charsize=3.
  for s=0,18-1 do begin
     w = where(str.number eq s,c)
     if c ge 2 then oplot,str[w].time,str[w].(pos[b]),color= (s mod 6) + 1,ps=3
  endfor
endfor
!p.multi=0
end



function rotate_to_gse,sat,earth,reverse=rev  ; only good for single points!
  earth_pos = reform(transpose(earth.rx))
  earth_vel = reform(transpose(earth.rv))
  l=crossp(earth_pos,earth_vel)
  s= -earth_pos
  rot = rot_mat(l,s)
  if keyword_set(rev) then rot = inverse(rot)
  pos = sat.rx # rot
  return,pos
end







function save_plot
plt = {d:!d, x:!x, y:!y, z:!z, p:!p}
return,plt
end


pro restore_plot,plt
wi,plt.d.window
!x=plt.x
!y=plt.y
!z=plt.z
end


function enter,default,prompt
val = default
read,val,prompt=prompt+' (default = "'+string(default)+'")? '
if not keyword_set(val) then val=default
return,val
end



;Begin Program....




common orbit_com,all

filename='orbit'

filename=enter('orbit','filename:')

tmax = 1200; days

if not keyword_set(all) then init_state
if not keyword_set(lim) then begin
window,0,xsize=500,ysize=325
  xlim,lim,-3,20
  ylim,lim,-8,8
  xlim,lim,-25,25
  ylim,lim,-25,25
  xlim,lim,-65,65
  ylim,lim,-35,35
  options,lim,'aspect',1
endif
if not keyword_set(dt) then  dt = .001d

nan=!values.d_nan
;burn={time:nan,sp_num:0,dvvec:[nan,nan,nan],dv:0.d,th:0.d,phi:0.d}

;deploy = replicate(burn,nsats)
;deploy.sp_num= indgen(nsats)
;deploy.time=1. + (10.d/60.d/24.d) * indgen(nsats)/4
;deploy.dv = 1.d
;deploy.th = 90.d
;deploy.phi = (indgen(nsats) mod 4)*90.d

;burn = deploy
;burn.dvvec[0] = burn.dv * sin(burn.th/!radeg) * cos(burn.ph/!radeg)
;burn.dvvec[1] = burn.dv * sin(burn.th/!radeg) * sin(burn.ph/!radeg)
;burn.dvvec[2] = burn.dv * cos(burn.th/!radeg)


a = dgen(100,range=[0,2*!dpi])
xe = cos(a)
ye = sin(a)
origin = (where(all.name eq 'Earth'))[0]

names0 = 'name time x v'
names1 = 'name time rx rv'
names2 = 'name orbit time apapsis periapsis period eccent inclin raan argp t_per'

names=names1
names=names2

pnames ='Apapsis Periapsis Inclin Argp RAAN'

nsat = n_elements(all)
cols = (indgen(nsat) mod 4)+1

si = where(strmid(all.name,0,1) eq 's',ns)
ms = ns*(ns-1)/2
if ns gt 1 then ind = intarr(2,ms)
m = 0
for i=0,ns-1 do for j= i+1,ns-1 do begin & ind(*,m) = [si(i),si(j)] & m=m+1 & end

t = 0.d
vec  = reform([all.x,all.v],6*nsat)
dvecdt = orbderiv(t,vec)

kb = 'r'

m0=10
m1=183 * 1000l
m2=1   * 1000l
alldat=0
sdat=0
counter=0l
index=0l
sindex=0l
;set_orbital_elements,all,t,reform(rvec[*,0,*]),reform(rvec[*,1,*])
set_orbital_elements,all
print_struct,all,tags=names
append=0
cr = string("15b)             ;a carriage return (no new line)
repeat begin
  rvec  = reform(vec,3,2,nsat)
  ori   = rvec(*,*,origin)
  ori   = reform(ori[*] # replicate(1.d,nsat),3,2,nsat)
  rstate = rvec - ori
  x = reform(rvec[0,0,*]) - rvec[0,0,origin]
  y = reform(rvec[1,0,*]) - rvec[1,0,origin]
  z = reform(rvec[2,0,*]) - rvec[2,0,origin]
  p1 = x
  p2 = y
  p1 = x
  p2 = z
;  p1 = (lim.xrange[0]-1) > x < (lim.xrange[1]+1)
;  p2 =  (lim.yrange[0]-1) > y < (lim.yrange[1]+1)

  if (counter mod m0) eq 0 then begin
    print,cr,t,'  ',format='(a,f9.4,a,$)'
    plots,p1,p2,psym=3,color=cols
  endif

  w = where((t ge all.t_per)  ,c)
  if c ne 0 then begin  ; test for each perigee
    set_orbital_elements,all,t,reform(rvec[*,0,*]),reform(rvec[*,1,*])
;    w = where((t ge all.t_per)  ,c)
    print,cr,format='(a,$)'
    for i=0,c-1 do begin
       s=all[w[i]]
       print,s.t_per,s.orbit,s.periapsis,s.name,form="(f8.4,'  P#',i0,f8.4,' (',a,')')"
       append_array,sdat,s,index=sindex
    endfor
  endif

  if (counter mod m1) eq 0 then begin
    set_orbital_elements,all,t,reform(rvec[*,0,*]),reform(rvec[*,1,*])
    print_struct,all,tags=names,file=filename+'.dat',append=append
  endif

  if (counter mod m2) eq 0 then begin
    set_orbital_elements,all,t,reform(rvec[*,0,*]),reform(rvec[*,1,*])
    append_array,alldat,all,index=index
  endif


;  if (counter mod m3) eq 0 then begin
;    dr0 = dr
;    dr = sqrt( total( reform( rvec[*,0,ind[0,*]] - rvec[*,0,ind[1,*]] )^2 , 1) )
;    dy = transpose([[dr0],[dr],[dr*nan]]
;    dx = transpose([[dr*0+t-dt],[dr*
;    oplot,counter+dr*0,
;  endif

  case kb of
    'r' : begin
       wi,0
       box,lim
       oplot,xe,ye
       end
    'w' :  begin
       r = sqrt(x^2 + y^2 + z^2)
       w = where(r lt 4.,nbad)
       if nbad gt 0 then p1[w] = !values.f_nan
       for m=0,ms-1 do begin
         i =ind(*,m)
         oplot,p1[i],p2[i]
       endfor
       end
    'd' : begin
       print
       set_orbital_elements,all ,t,reform(rvec[*,0,*]),reform(rvec[*,1,*])
       print_struct,all,tags=names
       end
    'p' : begin
       plt=save_plot()
       wi,1
       plot_struct2,sdat,names=pnames
       restore_plot,plt
       end
    'o' : begin
       plt=save_plot()
       wi,1
       nbods = n_elements(all)
       foo = reform(alldat[0:index-1],nbods,index/nbods)
       plot_struct,foo[2:nbods-1,*],names=pnames
       restore_plot,plt
       end
    's' : stop
    '0' : begin
       wi,0
       makegif,filename
       end
    '1' : begin
       plt=save_plot()
       wi,1
       makegif,filename+'_elem'
       restore_plot,plt
       end
    else:
  endcase
  vec = rk4(vec,dvecdt,t,dt,'orbderiv',/double)
  dvecdt = orbderiv(t,vec)
  counter=counter+1
  t = t+dt
  kb = get_kbrd(0)
endrep until (kb eq 'q') or (t ge tmax)
append_array,alldat,index=index,/done
append_array,sdat,index=sindex,/done
plt=save_plot()
wi,1
plot_struct2,sdat,names=pnames
makegif,filename+'_elem'
restore_plot,plt

save,alldat,sdat,file=filename+'.sav'


nbods = n_elements(all)
alldat = reform(alldat,nbods,index/nbods,/overwrite)
end

