;+
;FUNCTION:  read_orb_elem( filename)
;PURPOSE:
;   Reads data from an ascii file and puts data in an array of structures.
;CREATED BY: Davin Larson
;-

function read_orb_elem,filename,format=form,gm=gm,time0=time0,name=name

gm=0.d
if not keyword_set(filename) then filename=pickfile()

if not keyword_set(filename) then return,0

;f= orb_body()
if keyword_set(form) then f=form else   $
  f={time:0.d,   eccent:0.d,     periapsis:0.d,   inclin:0.d, $
                 raan:0.d, argp:0.d,   top:0.d,  $
                 m_motion:0.d,    m_anomaly:0.d,   phi :0.d,  $
                 semimajor:0.d,    apapsis:0.d,   period:0.d  $
                 , gm_center:0.d , name:'', relname:'', t_per:0.d} 

openr,lun,filename,/get_lun

ctype = bytarr(256)
ctype([32,9]) = 1                 ; white space
ctype(byte('.-0123456789')) =2    ; numbers

s = ''

max = 0l
rec = 0l

time = f.time
e= f.eccent    & q=f.periapsis  & i=f.inclin
lan= f.raan    & apf=f.argp     & top=f.top
n= f.m_motion  & ma=f.m_anomaly & ta=f.phi
a= f.semimajor & ad=f.apapsis   & per=f.period

readit = 0

while not eof(lun) do begin
  readf,lun,s
  bt = byte(s)
  nbt = n_elements(bt)
  j = 0
;  while (j lt nbt-1) and (ctype(bt(j)) eq 1) do j=j+1   ; skip white space

  if s eq '$$EOE' then readit = 0
  if readit then begin    ; numbers encountered
  
    f0 = f
    
    reads,s,time
    f0.time = time
    
    readf,lun,e,q,i
    f0.eccent=e   &   f0.periapsis=q   &   f0.inclin=i    
    
    readf,lun,lan,apf,top
    f0.raan=lan   &   f0.argp=apf   &   f0.top=top    
    
    readf,lun,n,ma,ta
    f0.m_motion=n   &   f0.m_anomaly=ma   &   f0.phi=ta    
  
    readf,lun,a,ad,per
    f0.semimajor=a   &   f0.apapsis=ad   &   f0.period=per    

    append_array,data,f0,index=index
    
  endif else begin
    if keyword_set(verbose) then print,s
    if strmid(s,0,12) eq 'GM of center' then GM = double(strmid(s,18,30))
    if strmid(s,0,11) eq 'Target body' then name = strtrim(strmid(s,18,24),2)
    if strmid(s,0,12) eq 'Coord center' then relname = strtrim(strmid(s,18,24),2)
  endelse
  
  if  (s eq '$$SOE') then readit = 1
endwhile

append_array,/done,data,index=index

free_lun,lun

;data.gm_center = gm / 6374.d^3
data.gm_center = gm 
data.name = name
data.relname = relname
data.t_per = data.time+data.top
;data.time = (data.time-2440587.5d)*3600.d*24.d


; warning: interpolation of t_per is not done properly and is a potential
; source of error!
if keyword_set(time0) then begin
  dt = data_type(data,/str)
  w = where(  ([0,0,0,0,1,1,0,0,0,0,0])(dt) )
  rdat = data[0]
  for i=0,n_elements(w)-1 do begin
    rdat.(w[i]) = interp(data.(w[i]),data.time,time0)
  endfor
  return,rdat
endif


return,data
end
