;PROCEDURE:  read_asc, data, filename
;PURPOSE:
;   Reads data from an ascii file and puts data in an array of structures.
;   Columns of data should be delimited by spaces.
;   Data is returned as an array of structures. The elements of the structure
;   correspond to the columns of the file.
;CALLING PROCEDURE:
;   read_ascii,data,'file.dat'
;KEYWORDS:
;   TAGS:  If set then the labels in the text line 
;      preceeding the data will be used for the default struct tag names.
;      (There should be one label per column of data)
;   FORMAT:  a structure that specifies the output format
;      of the data.  For example if the input file has the
;      following data:
;      Year Day  secs    Vx     Vy     Vz      N
;      1996 123  13.45  512.3  -10.3   10.5   5.3
;      the format could be specified as:
;   FORMAT={year:0,day:0,sec:0.d,v:fltarr(3),n:0.}
;   if this keyword is not specified then a default structure will be created.
;CREATED BY: Davin Larson


;pro read_asc,data,filename,format=f,verbose=verbose,tags=filetags,time=time $
;  ,no_conv_time=no_conv_time


function read_orbits,filename,format=f

if not keyword_set(filename) then filename=pickfile()

if not keyword_set(filename) then return,0

if not keyword_set(f) then $
   f=orb_body()
;  f={time:0.d,   rx:dblarr(3),  rv:dblarr(3), owlt:0.d, r:0.d, dr_dt:0.d}

openr,lun,filename,/get_lun

ctype = bytarr(256)
ctype([32,9]) = 1                 ; white space
ctype(byte('.-0123456789')) =2    ; numbers

s = ''

max = 0l
rec = 0l

time = f.time
d3 =dblarr(3)

readit = 0

while not eof(lun) do begin
  readf,lun,s
  bt = byte(s)
  nbt = n_elements(bt)
  j = 0
;  while (j lt nbt-1) and (ctype(bt(j)) eq 1) do j=j+1   ; skip white space

  if s eq '$$EOE' then readit = 0
  if readit then begin    ; numbers encountered
  
    f0 = f
    
    reads,s,time
    f0.time = time
    
    readf,lun,d3
    f0.rx=d3
    
    readf,lun,d3
    f0.rv=d3
    
    readf,lun,d3
;    f0.owlt=d3[0]
    f0.r=d3[1]
;    f0.dr_dt=d3[2]
    
    append_array,data,f0,index=index
    
  endif else begin
    if keyword_set(verbose) then print,s
  endelse
  
  if  (s eq '$$SOE') then readit = 1
endwhile

append_array,/done,data,index=index

free_lun,lun

;data.time = (data.time-2440587.5)*3600.d*24.d

return,data
end
