;+
;PROCEDURE:	scat_plot, xname, yname
;PURPOSE:
;   Produces a scatter plot of selected tplot variables.
;   Colors are scaled according to zname, if present
;INPUTS:	
;	xname:	xvariable name
;	yname:	yvariable name
;	zname: 	if present, color variable name
;KEYWORDS:	
;       TRANGE:  two element vector giving start and end time.
;	limits:		a structure with plotting keywords
;
;CREATED BY:	Davin Larson
;LAST MODIFICATION:	@(#)scat_plot.pro	1.10 96/10/14
;-
;	begin_time:	time at which to start plot
;	end_time:	time at which to end plot

pro scat_plot,xname,yname, zname,$
   xvalues = x, $
   yvalues = y, $
   tvalues = time, $
   begin_time = t0,  $
   end_time  = t1,  $
   trange=trn,  $
   randomize=randomize, $
   tsource = tsource, $
   limits = limits


get_data,xname,data=xdata,alim=xlim
get_data,yname,data=ydata,alim=ylim



if n_elements(xdata) eq 0 then message,'No data associated with: '+xname

prompt = 'Which dimension of '+xname+'? '
if dimen2(xdata.y) gt 1 then read, nx, prompt = prompt else nx = 0
prompt = 'Which dimension of '+yname+'? '
if dimen2(ydata.y) gt 1 then read, ny, prompt = prompt else ny = 0




ts = keyword_set(tsource) ? tsource : 1
case ts of
  3:  time=zdata.x
  2:  time=ydata.x
 else:  time=xdata.x
endcase



time = time(where(finite(time)))

if keyword_set(trn) then begin
   trange = time_double(trn)
   time = time(where( (time le trange(1)) and (time gt trange(0)) ))
endif

x = interp(xdata.y(*,nx),xdata.x,time)
y = interp(ydata.y(*,ny),ydata.x,time)

ztitle=''

;get plotting stuff

three = n_params() eq 3
if three then begin
	get_data,zname,data=zdata,alim=zlim
	prompt = 'Which dimension of '+zname+'? '
	if dimen2(zdata.y) gt 1 then read, nz, prompt = prompt else nz = 0
	z = interp(zdata.y(*,nz),zdata.x,time)
	zrange = struct_value(zlim,'yrange',default=minmax(z))
	zrange = struct_value(alimits,'zrange',default=zrange)
	zlog = struct_value(zlim,'ylog',default=0)
	zlog = struct_value(limits,'zlog',default=zlog)
	ztitle = struct_value(zlim,'ytitle',defa=zname)
	ztitle = struct_value(limits,'ztitle',defa=ztitle)
	colors = bytescale(z, range = zrange,  log = zlog)
endif else begin
	colors = replicate(!p.color, dimen1(time))
endelse


title = string(/print,time_string(minmax(time)))

xtitle = xname
ytitle = yname
str_element,xlim,'ytitle',xtitle
str_element,ylim,'ytitle',ytitle
str_element,zlim,'ytitle',ztitle



plotstuff={xtitle:xtitle,ytitle:ytitle,ztitle:ztitle,title:title,psym:3}
extract_tags,plotstuff,ylim,tags=['yrange','ylog','ystyle']
str_element,xlim,'yrange',xrange
str_element,xlim,'ylog',xlog
str_element,xlim,'ystyle',xstyle
str_element,plotstuff,'xrange',xrange,/add
str_element,plotstuff,'xlog',xlog,/add
str_element,plotstuff,'xstyle',xstyle,/add



extract_tags,plotstuff,limits

box,plotstuff,x,y
if keyword_set(randomize) then begin
  r = randomu(randomize,n_elements(x))
  s = sort(r)
  x=x[s]
  y=y[s]
  colors=colors[s]
endif

plots,x,y,color=colors,_extra=plotstuff,noclip=0

if three then draw_color_scale,range=zrange,log=zlog,title=ztitle

;mplot,x,y,limit=plotstuff
;if debug() then stop

;plot,x,y,xtitle=xtitle, ytitle=ytitle, title = title, /nodata, _EXTRA = plotstuff
;plots,x,y,color=colors, psym = psym, noclip = 0

return
end
