


pro shock_detect,sd,append=append,max=max,  $
  trange=trange,verbose=verbose,do_plots=do_plots

sda =shock_struct()

get_data,'Np',time,np
get_data,'Vp',time,vp
get_data,'Tp',time,tp
magf = data_cut('wi_B3',time)
bmag = sqrt(total(magf^2,2))

dt   = (shift(time,-1) - shift(time,1))
dvp  = (shift(vp,-1,0) - shift(vp,1,0))
dvm  = sqrt(total(dvp^2,2))
dtp  = (shift(tp,-1) - shift(tp,1))
dnp  = (shift(np,-1) - shift(np,1))
db   = (shift(bmag,-1) - shift(bmag,1))

good = dt lt 8.
good = good and dt gt 0
good = good and abs(db)/bmag gt .2
good = good and dvm gt 30
good = good and np gt .1
good = good and abs(dnp)/np gt .2
good = good and abs(dtp)/tp gt .2
good[0] = 0
good[n_elements(good)-1] = 0
good = good and (shift(good,1) eq 0)

if keyword_set(trange) then $
   good = good and (time ge trange[0]) and (time lt trange[1])

w=where(good,c)

;help,w
;timebar,time[w]

if c eq 0 then return
store_data,'bars',data={x:time[w]}

if not keyword_set(max) then max = 10
if c gt max then begin
  message,/info,'Warning! Too many shocks found!'
  c = max
endif 

asdat = replicate(sdat,c)
if keyword_set(verbose) then print,c,' shock(s) found'
n = n_elements(sdat.dat)
for i=0,c-1 do begin
   wi = w[i]
   ind = wi+indgen(n)-n/2
   asdat[i].ts = time[wi]
   asdat[i].scpos = data_cut('wi_pos',time[wi])
   asdat[i].dat.t  = time[ind]
   asdat[i].dat.n  = np[ind]
   asdat[i].dat.v  = transpose(vp[ind,*])
   asdat[i].dat.tp = tp[ind]
   asdat[i].dat.b  = transpose(magf[ind,*])
   if keyword_set(do_plots) then begin
      tplot,'wi_B3_mag wi_B3_th wi_B3_phi Np Vp Tp RVVp',trange= time[wi]+[-150,150]
;      timebar,time[w]
      makegif,'IPS_'+time_string(time[wi],form=2)     
   endif
   
endfor


if keyword_set(append) and size(/type,sd) eq 8 then sd = [sd,asdat] else sd = asdat

return
end


pro   shock_detect_all,trange,ndays=ndays,zbuff=zbuff,sda=sda


trfull = floor(minmax(time_double(trange))/86400d) * 86400d
n = round((trfull[1]-trfull[0])/86400.d) +1

if not keyword_set(ndays) then ndays =1
sda = 0
verbose = 1
if keyword_set(zbuff) then begin
   oldname= !d.name
   set_plot,'z'
   device,set_resol=[640,800]
endif

tlabel = time_string(trfull,/date,form=2)

for i=0,n-1,ndays do begin
  t = trfull[0] + i * 86400d
  tr = [t,t+ndays*86400d]
  timespan,tr 

  load_wi_h0_mfi,/pol
  load_wi_or,/var
  load_wi_pm2_3dp
  
  shock_detect,sda,/append,trange=tr,verbose=verbose,/do_plot
;  if i mod ndays eq 0 then $
     save,sda,file='IPSD_'+tlabel[0]+'_'+tlabel[1]+'.dat'
  
endfor

save,sda,file='IPSD_'+tlabel[0]+'_'+tlabel[1]+'.dat'

if keyword_set(zbuff) then set_plot,oldname

  
end


pro fill_shock_region,sda

n = n_elements(sda)
ind1 = indgen(48)
ind2 = indgen(48) +52
for i=0,n-1 do begin
  sd = sda[i]
  if  sd.r1.tr[0] eq 0 then begin
  bmag = sqrt(total(sd.dat.b^2,1))
  vmag = sqrt(total(sd.dat.v^2,1))
  sd.r1.n= average(sd.dat[ind1].n,stdev=dn1,/nan)
  sd.r2.n= average(sd.dat[ind2].n,stdev=dn2,/nan)
  sd.r1.tp= average(sd.dat[ind1].tp,stdev=dt1,/nan)
  sd.r2.tp= average(sd.dat[ind2].tp,stdev=dt2,/nan)
  sd.r1.v= average(sd.dat[ind1].v,2,stdev=dv1,/nan)
  sd.r2.v= average(sd.dat[ind2].v,2,stdev=dv2,/nan)
  sd.r1.b= average(sd.dat[ind1].b,2,/nan)
  sd.r2.b= average(sd.dat[ind2].b,2,/nan)
  
  if sd.r1.tp gt sd.r2.tp then begin
       sd.order=1
       r = sd.r1
       sd.r1 = sd.r2
       sd.r2 = r
  endif else sd.order = 0
  sda[i] = sd
  endif
endfor

end


pro shock_detect2_test,sda,ok
dir = '~davin/data/IPS_plots/'
n = n_elements(sda)
ok = intarr(n)
for i=0,n-1 do begin
  sd = sda[i]
  bmag = sqrt(total(sd.dat.b^2,1))
  vmag = sqrt(total(sd.dat.v^2,1))
  n1= average(sd.dat[ 0:48].n,stdev=dn1,/nan)
  n2= average(sd.dat[52:99].n,stdev=dn2,/nan)
  t1= average(sd.dat[ 0:48].t,stdev=dt1,/nan)
  t2= average(sd.dat[52:99].t,stdev=dt2,/nan)
  v1= average(sd.dat[ 0:48].v,2,stdev=dv1,/nan)
  v2= average(sd.dat[52:99].v,2,stdev=dv2,/nan)
  b1= average(sd.dat[ 0:48].b,2,/nan)
  b2= average(sd.dat[52:99].b,2,/nan)
  ang12 = acos(total(b1*b2)/norm(b1)/norm(b2)) *!radeg
  mm_n = minmax(sd.dat.n)
  mm_b = minmax(bmag)
  mm_v = minmax(vmag)
  ok[i] = 1
;  ok[i] = ok[i] and ang12 lt 60.
  ok[i] = ok[i] and mm_n[0] gt .5
  ok[i] = ok[i] and mm_v[0] gt 250.
  ok[i] = ok[i] and mm_v[1] lt 1000.
  ok[i] = ok[i] and mm_b[1]/mm_b[0] lt 5
  ok[i] = ok[i] and mm_n[1]/mm_n[0] lt 5
  dn = abs(n1-n2)/(n1+n2)*2
  
  ok[i] = ok[i] and dn gt .20 
  ok[i] = ok[i] and dn1 lt .3*dn*n1 
  ok[i] = ok[i] and dn2 lt .3*dn*n2 
;  ok[i] = ok[i] and abs(n1-n2)/(n1+n2) gt .15 
  if 1  then begin; and ok[i]
     fname = dir+'IPS_'+time_string(sd.ts,form=2)+'.gif' 
     f_ok = query_gif(fname)
     if(f_ok) then begin     
       read_gif,fname,im,r,g,b
       tv,im
       print,i
       k = get_kbrd(1)
       if k eq 'y' then sda[i].ok = 1
       if k eq 'n' then sda[i].ok = 0
       if k eq 'q' then return
     endif else erase
  endif

endfor
w = where(ok,c)
print,c,' out of ',n,' are ok'

end

