pro synoptic_mag, file,limits=gkmlim,time=t1,rotnum=rotnum,values=z

common synoptic_com, path, filter

if not keyword_set(path) then path = '~davin/dat/solar/kittpeak'
if not keyword_set(filter) then filter = '*.fits'


if data_type(file) ne 7 then $
  file=pickfile(path=path,get_path=path,filter=filter,/read)

if not keyword_set(file) then return


if data_type(gkmlim) ne 8 then begin
  gkmlim=0
  zlim,gkmlim,-50,50
  xlim,gkmlim,0,360
  ylim,gkmlim,0,180
  options,gkmlim,'x_no_interp',1
  options,gkmlim,'y_no_interp',1
  options,gkmlim,'xmargin',[10.,10.]
endif


image = readfits(file,h,/nosc)
bscale = float(sxpar(h,'BSCALE'))
if bscale eq 0 then bscale = 1.
bzero = float(sxpar(h,'BZERO'))
nx = dimen1(image)
ny = dimen2(image)

irafname = sxpar(h,'IRAFNAME')
numbs = byte('0123456789')
bmap = replicate(32b,256)
bmap(numbs) = numbs
rotnum = fix(string(bmap(byte(irafname))))

time=dblarr(36)
tstr = time_struct(0.d)
lon=fltarr(36)

for i = 0,35 do begin
   dstr = string(i+1,format="('DATE',i4.4)")
   lstr = string(i+1,format="('LONG',i4.4)")
   d = floor(sxpar(h,dstr))
   tstr.year  = d mod 100 + 1900  &   d = d / 100
   tstr.date  = d mod 100         &   d = d / 100
   tstr.month = d mod 100
   time(i) = time_double(tstr)
   lon(i) = sxpar(h,lstr)
endfor
w = where(lon,c)
lon = lon(w)
time = time(w)


for i=0,c-1 do  print,time_string(time(i)),lon(i)

w = where(lon lt 270 and lon gt 90)
t1 = interp(time(w),lon(w),180.)


;t1= min(time)

p={x0:0.,y0:0.,r0:01.}

x = (findgen(nx)+.5-p.x0)/p.r0
y = (findgen(ny)+.5-p.y0)/p.r0
xt = x # replicate(1.,ny)
yt = replicate(1.,ny) # y
rt = sqrt(xt^2 + yt^2)
z = image * bscale + bzero
bad = where(rt gt .99 and z lt -200,count)
if count ne 0 then z(bad) = !values.f_nan
print,bscale,min(z,/nan),max(z,/nan)
options,gkmlim,'title','Kitt Peak synoptic map '+irafname+' '+time_string(t1[0])

options,gkmlim,'position',plot_pos(lim=gkmlim,/top)
specplot,x,y,z,lim=gkmlim
levels = (findgen(10)-4.5)*200
;box,gkmlim & contour,z,x,y,lev=levels,/over,/fill
;contour,z,x,y,lev=levels,/over,col=0

;a = findgen(360)/360.*2*!pi
;oplot,sin(a),cos(a)


end
