function vector,n
if not keyword_set(n) then n=1
return, replicate({Vec_cart,x:0.d,y:0d,z:0d},n)
end

function sym_tensor,n
if not keyword_set(n) then n=1
return, replicate({STens_cart,xx:0d,yy:0d,zz:0d,xy:0d,yz:0d,zx:0d},n)
end

function tensor,n
if not keyword_set(n) then n=1
return, replicate({Tens_cart,xx:0d,yy:0d,zz:0d,xy:0d,xz:0d,yz:0d,yx:0d,zx:0d,zy:0d},n)
end


function partial,f,r,dim
if dim eq 0 then par = deriv(r.x,f)
if dim eq 1 then par = 0
if dim eq 2 then par = 0

return,par
end
function vector_sum,V1,V2
Vs=V1
vs.x = v1.x + v2.x
vs.y = v1.y + v2.y
vs.z = v1.z + v2.z
return,vs
end


function vmult,V,m
Vm = V
Vm.x = m*V.x
Vm.y = m*V.y
Vm.z = m*V.z
return,Vm
end

function cross_prod,A,B,mult=m
C = A
if not keyword_set(m) then m=1
C.x = m*( A.y * B.z - A.z * B.y)
C.y = m*( A.z * B.x - A.x * B.z)
C.z = m*( A.x * B.y - A.y * B.x)
return,C
end


function stens_cross_prod,M,B,mult=mu
C = M
if not keyword_set(mu) then mu=1
C.xx = mu*2*( M.xy*B.z - M.zx*B.y )
C.yy = mu*2*( M.yz*B.x - M.xy*B.z )
C.zz = mu*2*( M.zx*B.y - M.yz*B.x )
C.xy = mu*( M.zx*B.x - M.yz*B.y + (M.yy-M.xx) * B.z)
C.yz = mu*( M.xy*B.y - M.zx*B.z + (M.zz-M.yy) * B.x)  ; check this!
C.zx = mu*( M.yz*B.z - M.xy*B.x + (M.xx-M.zz) * B.y)  ; check this!
return,C
end




function dot_prod,A,B,mult=m
if not keyword_set(m) then m=1
dot = (A.x*B.x+A.y*B.y+A.z*B.z) * m
return,dot
end


function div, V,r,mult=m
if not keyword_set(m) then m=1
dVx = partial(V.x,r,0)
dVy = partial(V.y,r,1)
dVz = partial(V.z,r,2)
return,m*(dVx+dVy+dVz)
end


function divt, T
DT = vector(size(/dimen,T))
DT.x = partial(T.xx,r,0)  + partial(T.xy,r,1) + partial(T.xz,r,2)
DT.y = partial(T.yx,r,0)  + partial(T.yy,r,1) + partial(T.yz,r,2)
DT.z = partial(T.zx,r,0)  + partial(T.zy,r,1) + partial(T.zz,r,2)
return,dt
end

function divst, sT
DT = vector(size(/dimen,sT))
DT.x = partial(sT.xx,r,0)  + partial(sT.xy,r,1) + partial(sT.zx,r,2)
DT.y = partial(sT.xy,r,0)  + partial(sT.yy,r,1) + partial(sT.yz,r,2)
DT.z = partial(sT.zx,r,0)  + partial(sT.yz,r,1) + partial(sT.zz,r,2)
return,dt
end

function curl, A,r, mult=m
B=A
B.x = m* (partial(A.z,r,1) - partial(A.y,r,2) )
B.y = m* (partial(A.x,r,2) - partial(A.z,r,0) )
B.z = m* (partial(A.y,r,0) - partial(A.x,r,1) )
return,B
end

function grad, f,r, mult=m
if not keyword_set(m) then m=1
g = vector(size(/dimen,f))
g.x = m* partial(f,r,0)
g.y = m* partial(f,r,1)
g.z = m* partial(f,r,2)
return,g
end

pro def_constants

mu0 = 4*!dpi*1e-7
C = 2.99792d8
eps0 = 1/C^2/mu0
Mp    = 1.6726d-27   ; kg
Me  = Mp /1836  ; kg
Mt = Mp+Me  ; kg
q_e = 1.6022d-19    ; C
J_ev =  q_e

const={ $
    c:c  $
   ,mu0: mu0  $
   ,eps0 : eps0  $
   ,Mp    : Mp   $
   ,Me  : Mp /1836  $
   ,Mt : Mp+Me     $
   ,q_e : q_e  $
   }

defsysv,'!cc',const
end


function dS_dt,S
dS = S
ds.B = curl(S.E,S.r,mult=-1)

ds.E =  Vector_sum(  curl(s.B,s.r,mult=!cc.c^2),  vmult(S.J,-1/!cc.eps0) )

return,ds
end

;pro plasma1d

;mu0 = 4*!dpi*1e-7
;C = 2.99792d8
;eps0 = 1/C^2/mu0
;Mp    = 1.6726d-27   ; kg
;Me  = Mp /1836  ; kg
;Mt = Mp+Me  ; kg
;q_e = 1.6022d-19    ; C
;J_ev =  q_e



function init_state

L = 10000
n = 10000
dim = [n]

scaler=0d
vec=vector()
stens=sym_tensor()
state ={state2,r:vec, B:vec, E:vec, rho:scaler, J:vec, mn:scaler, V:vec,  P:stens, vec1:[0.,0.,0.] }

s=replicate(state,dim)

s.r.x = dindgen(n)/n * L

;Initial conditions
s.B.x = 0
s.B.y = 10e-9 * ( erfc(2*sin(s.r.x/L*2*!dpi))/2+.1 )
s.B.z = 0

;gamma_f = sym_tensor(dim)
;gamma_f.xx = (-B.x^2+B.y^2+B.z^2 ) /2/!cc.mu0   ; + electric field part
;gamma_f.yy = (+B.x^2-B.y^2+B.z^2 ) /2/!cc.mu0   ; + electric field part
;gamma_f.zz = (+B.x^2+B.y^2-B.z^2 ) /2/!cc.mu0   ; + electric field part
;gamma_f.xy = ( -2*B.x*B.y )        /2/!cc.mu0   ; + electric field part
;gamma_f.xz = ( -2*B.x*B.z )        /2/!cc.mu0   ; + electric field part
;gamma_f.yz = ( -2*B.y*B.z )        /2/!cc.mu0   ; + electric field part


s.J = curl(s.B,s.r,mult=1/!cc.mu0)
s.E = cross_prod(s.B,s.V)  ; + other ohms law terms
s.rho = !cc.eps0 * div(s.E,s.r)

dens = 5d *1e6
s.mn = !cc.Mt * dens


s.p.xx = (-s.b.x^2 + s.b.y^2 + s.b.z^2)/2/ !cc.mu0 - s.mn * s.v.x^2




FF = cross_prod(s.J,s.B)
FF.x = FF.x + s.rho * s.E.x
FF.y = FF.y + s.rho * s.E.y
FF.z = FF.z + s.rho * s.E.z



j_ev = !cc.q_e
temperature = 1000 * J_ev   ; Joules

p = 100 * J_ev
s.p.xx = p
s.p.yy = p
s.p.zz = p


gamma_k = s.p
gamma_k.xx = s.p.xx + s.mn * s.V.x * s.V.x
gamma_k.yy = s.p.yy + s.mn * s.V.y * s.V.y
gamma_k.zz = s.p.zz + s.mn * s.V.z * s.V.z
gamma_k.xy = s.p.xy + s.mn * s.V.x * s.V.y
gamma_k.zx = s.p.zx + s.mn * s.V.x * s.V.z
gamma_k.yz = s.p.yz + s.mn * s.V.y * s.V.z


!p.multi = [0,1,3]
plot,s.r.x,s.b.y*1e9, ytitle='By'
plot,s.r.x,s.j.z, ytitle='Jz'
plot,s.r.x,FF.x, ytitle='(J X B)x'

return,s
end


def_constants
s=init_state()
end