pro tz
  ctime,t,y,vname=vname
  if vname[0] eq vname[1] then ylim,vname[0],y
  tplot,trange=t,vname[0]

end

pro update,range=range,rawfile=rawfile
if keyword_set(rawfile) then read_sstgse_dat,rawfile
if not keyword_set(range) then range=10.
tmax = systime(1)-8*3600d
get_data,'Ctot',data=d
tmax = max(d.x)
tlimit,tmax + [-range,0.0]*60.
end


function  find_peak,d,cbins,window = w
peak = igauss()
if not keyword_set(cbins) then cbins = dindgen(n_elements(s))
t = total(d)
avg = total(d * cbins)/t
sdev = sqrt(total(d*(cbins-avg)^2)/t)
peak.c=t
peak.x0=avg
peak.w=sdev
return,peak
end



function find_peaks,dat
peak={C:!values.d_nan,x0:!values.d_nan,w:!values.d_nan}
if not keyword_set(dat) then return,peak
channels = dindgen(n_elements(dat))
plot,dat,xrange=[0,130],psym=10
nsm = 3
roiw = 7

thresh = 2.
sdat = (smooth(float(dat),nsm))
;oplot,sdat,col=6
deriv = sdat - shift(sdat,1)
;oplot,deriv,col=3
pks = where(deriv ge 0 and shift(deriv,-1) lt 0 and sdat gt thresh,np)
print,pks
ind = indgen(roiw)-roiw/2
peaks=replicate(peak,np)
for i=0,np-1 do begin
   d = dat[ind+pks[i]]
   c = channels[ind+pks[i]]
   oplot,c,d,col= 1,psym=10
   t = total(d)
   avg = total(d*c)/t
   sdev = sqrt(total(d*(c-avg)^2)/t)
   peak.c=t
   peak.x0=avg
   peak.w=sdev
   peaks[i]=peak
   print,i,peak.c,peak.x0,peak.w,peak.w*2.35
endfor

return,peaks
end


function timeshift,t,parameter=par
if not keyword_set(par) then begin
   if keyword_set(t) then ref= min(t) else ref = systime(1)
   par={func:'timeshift', t0:0d,  s:.001d, reftime:long(ref) }
endif
if n_params() eq 0 then return,par
return, (t-par.t0-par.reftime)*(1+par.s) + par.t0 + par.reftime
end

pro plot_tshift,ta
wi,1
n=n_elements(ta)
ta1=reform(ta,2,n/2)
t0=reform(ta1[0,*])
t1=reform(ta1[1,*])
plot,t0-min(t0),t1-t0,/ps
wset,0
timebar,t0,var='AO0'
timebar,t1,var='C'
end

function cal_int_pulser,ts
if not keyword_set(ts) then ctime,ts,np=2
ds = tsample('dsteady',ts,time=t1)
c = tsample('C',ts,time=t2)
wstart = where(ds gt 0,nw1)  & wstart = reform(wstart,2,nw1/2) & wstart = wstart[0,*]
wstop  = where(ds lt 0,nw2)  & wstop  = reform(wstop ,2,nw2/2) & wstop  = wstop[1,*]
nw = nw1/2
pk = replicate(find_peaks(),4)
rdat = replicate({t:0d,ep:0d,ip:0d,ipc:0b,map:0b,pks:pk},nw)
pval =indgen(nw) * 1024
pval[0]= 512
c[*,0] = 0
for i = 0 ,nw-1 do begin
    i1= wstart[i]
    i2= wstop[i]
    s = average(c[i1:i2,*],1)
    s[0:1] = 0
    timebar,t1[[i1,i2]]
    print,time_string(t1[i1])
    rdat[i].pks = find_peaks(s)
endfor
foo=rdat[2:3].pks[1]
rdat[2:3].pks[1]=rdat[2:3].pks[0]
rdat[2:3].pks[0] = foo
rdat.ep = pval
return,rdat
end





function cal_pulser,ts,interval=dt
if not keyword_set(ts) then ctime,ts
e_pulser = data_cut('AO0',ts)
maps     = byte(data_cut('hkp-map',ts)+.5)
plsr     = byte(data_cut('hkp-plsr',ts)+.5)
if not keyword_set(dt) then dt = 1.
ns = n_elements(ts)
pk = find_peak([1])
rdat = replicate({t:0d,ep:0d,ip:0d,ipc:0b,map:0b,pk1:pk,pk2:pk},ns)
rdat.t = ts
rdat.ep = e_pulser
rdat.map = maps
rdat.ipc = plsr
wi,1
for i= 0,ns-1 do begin
   bins = sstmaps(maps[i],dbins=dbins)
   s = tsample(/average,'C',ts[i]+dt*[-1,1])
   s[0] =0
   pk = find_peak(s,bins+dbins/2.)
   rdat[i].pk1 = pk
   plot,s,ps=10
   fit,bins,s,func='igauss',param=pk
   f = igauss(bins,param=pk)
   oplot,f,ps=1
   ;stop
   rdat[i].pk2=pk
endfor

return,rdat
end


pro calc_stats,var
if not keyword_set(var) then var='C'
get_data,var,data=d
d.y[*,0] = 0
st=get_stats(d.y,d.v)
store_data,var+'_tot' ,data={x:d.x,y:st[*,0]}
store_data,var+'_ctr' ,data={x:d.x,y:st[*,1]}
store_data,var+'_sig' ,data={x:d.x,y:st[*,2]}
end



pro I_vs_T_plot
plot,[-100,60],[0,30],/nodat,xtitle='Internal Themistor Temp',ytitle='Current (mA)
ctime,tr
ip1 = tsample('I_Preg1',tr,time=t)
temp1 = data_cut('Tint1',t)
ip2 = tsample('I_Preg2',tr,time=t)
temp2 = data_cut('Tint2',t)
in1 = tsample('I_Nreg1',tr,time=t)
temp1 = data_cut('Tint1',t)
in2 = tsample('I_Nreg2',tr,time=t)
temp2 = data_cut('Tint2',t)

oplot,temp2,ip2,col=6,psym=3
oplot,temp1,ip1,col=2,psym=3
oplot,temp2,in2,col=6,psym=3
oplot,temp1,in1,col=2,psym=3
end


pro noise_vs_T_plot,tr
if not keyword_set(tr) then ctime,tr
temp1 = tsample('Tint1',tr,time=t1)
ns = data_cut('Nse_sig_sm',t1)

temp2 = tsample('Tint2',tr,time=t2)
na = data_cut('Nse_avg_sm',t2)

!p.multi = [0,2,2]
col = get_colors('bgr')
str=strcompress(/rem,indgen(4))
for g=0,3 do begin
plot,[-80,60],[-2,4],/nodat,xtitle='Internal Themistor Temp',ytitle='Noise (bins) group '+str[g],/xstyle,/ystyle
oplot,g le 1 ? temp1 : temp2,ns[*,0+g*3],col = col[0],psym=3
oplot,g le 1 ? temp1 : temp2,ns[*,1+g*3],col = col[1],psym=3
oplot,g le 1 ? temp1 : temp2,ns[*,2+g*3],col = col[2],psym=3
oplot,g le 1 ? temp1 : temp2,na[*,0+g*3],col = col[0],psym=3
oplot,g le 1 ? temp1 : temp2,na[*,1+g*3],col = col[1],psym=3
oplot,g le 1 ? temp1 : temp2,na[*,2+g*3],col = col[2],psym=3
endfor
!p.multi=0
end


pro test_poisson,tr
wi,0
s = tsample('Clow',tr,/silent)
p = poisson()
p.avg = average(s)
n = n_elements(s)
p.h = n
wi,2
h = histbins(s,xb,bins=1)
plot,xb,h,psym=10;,col=6
pois = poisson(xb,para=p)
;oplot,xb,pois,ps=1
plotxyerr,xb,pois,xb*0+.5,sqrt(pois),col=6,/overpl
printdat,p
w = where(s le p.avg+4*sqrt(p.avg),nw)
print,total(s[w])/nw,'+/-',sqrt(total(s[w]))/nw
if nw ne n then print,"WARNING",strtrim(n-nw),' POINTS removed'
end


pro cp_spectra,t
if not keyword_set(t) then ctime,t
n= n_elements(t)/2
t = reform(t,2,n)
xlim,lim,0,50
ylim,lim,0,1
colors = [1,2,3,4,5,6,1,2,3,4,5,6]
plot,/nodata,_extra=lim,[0,1]
for i=0,n-1 do begin
    d= tsample('C',t[*,i],/average)
    oplot,d,color=colors[i mod n_elements(colors)]
endfor
end





pro load_dap,dapfiles

if  not keyword_set(dapfiles) then begin
   dapfiles = dialog_pickfile(filt='*.dap',/multi)
   if n_elements(dapfiles) gt 1 then    dapfiles = dapfiles(sort(dapfiles))
   if not keyword_set(dapfiles) then return
endif

if not keyword_set(dap) then begin
   dap = read_daq(dapfiles[0])
   if not keyword_set(dap) then return
   for i =1,n_elements(dapfiles)-1 do   dap =  [dap,read_daq(dapfiles[i])]
   dt = dap.time -shift(dap.time,1)
   w = where(dt gt 4,nw)
   if nw gt 1 then dap[w].data = !values.f_nan
endif

;timespan,minmax(dap.time)

   ;printdat,dap
   store_data,'I_Preg1',data={x:dap.time,y:  dap.data[0]/.0031}, dlim={colors:'b'}
   store_data,'I_Preg2',data={x:dap.time,y:  dap.data[1]/.0031}, dlim={colors:'r'}
   store_data,'I_Nreg1',data={x:dap.time,y: -dap.data[2]/.0062}, dlim={colors:'b'}
   store_data,'I_Nreg2',data={x:dap.time,y: -dap.data[3]/.0062}, dlim={colors:'r'}
   store_data,'I_Vreg',data='I_Preg2 I_Preg1 I_Nreg2 I_Nreg1',dlim={yrange:[0,30.],ystyle:1,ytitle:'Sensor Current (mA)'}

   V_bias = dap.data[10]
   V_rint1 = dap.data[4]
   V_rint2 = dap.data[5]
   V_rext1 = dap.data[8]
   V_rext2 = dap.data[9]

   w = where(v_bias lt 3.8,nw)
   if nw gt 0 then v_bias[w] = !values.f_nan

   dv = V_bias-4.0
   V_bias = V_bias - dv
   V_rint1= V_rint1 + .7*dv
   V_rint2= V_rint2 + .7*dv
   V_rext1= V_rext1 - dv
   V_rext2= V_rext2 - dv

   w = where(V_rext1 lt .1,nw)
   if nw gt 0 then V_rext1[w] = !values.f_nan
   w = where(V_rext2 lt .1,nw)
   if nw gt 0 then V_rext2[w] = !values.f_nan
   w = where(V_rint1 lt .1,nw)
   if nw gt 0 then V_rint1[w] = !values.f_nan
   w = where(V_rint2 lt .1,nw)
   if nw gt 0 then V_rint2[w] = !values.f_nan

   store_data,'V_Rext1',data={x:dap.time,y: V_rext1}, dlim={colors:'b'}
   store_data,'V_Rext2',data={x:dap.time,y: V_rext2}, dlim={colors:'r'}
   store_data,'V_Rext',data='V_Rext1 V_Rext2'
   store_data,'V_Rint1',data={x:dap.time,y: V_rint1}, dlim={colors:'b'}
   store_data,'V_Rint2',data={x:dap.time,y: V_rint2}, dlim={colors:'r'}
   store_data,'V_R',data='V_Rint1 V_Rint2 V_Rext1 V_Rext2 V_atten',dlim={yrange:[0,5],ystyle:1}


   if not keyword_set(pti1) then begin
      pti1 = therm_temp2()
      pti1.r1=10000.
      pti1.xmax=2.431
   endif
   if not keyword_set(pti2) then begin
      pti2 = therm_temp2()
      pti2.r1=10000.
      pti2.xmax=2.444
   endif

   Tint1 = therm_temp2(V_rint1,param=pti1)
   Tint2 = therm_temp2(V_rint2,param=pti2)
   store_data,'Tint1',data={x:dap.time,y: Tint1}, dlim={colors:'b'}
   store_data,'Tint2',data={x:dap.time,y: Tint2}, dlim={colors:'r'}
   store_data,'Tint',data='Tint1 Tint2'



   store_data,'I_Heater1',data={x:dap.time,y: dap.data[6]}, dlim={colors:'b',yrange:[-.05,.35],ystyle:1}
   store_data,'I_Heater2',data={x:dap.time,y: dap.data[7]}, dlim={colors:'r',yrange:[-.05,.35],ystyle:1}
   store_data,'I_Heater',data='I_Heater2 I_Heater1'        ,dlim={yrange:[-.05,.35],ystyle:1,panel_size:.3}


   pt1 = therm_temp2()  & pt1.r1=50000.   & pt1.xmax=1.0
   pt2 = pt1
   Text1 = therm_temp2(V_rext1/V_bias,param=pt1)
   Text2 = therm_temp2(V_rext2/v_bias,param=pt2)
   store_data,'Text1',data={x:dap.time,y: Text1}, dlim={colors:'b'}
   store_data,'Text2',data={x:dap.time,y: Text2}, dlim={colors:'r'}
   store_data,'Text',data='Text1 Text2'

   store_data,'V_atten',data={x:dap.time,y: dap.data[10]}
   store_data,'V_mux',  data={x:dap.time,y: dap.data[11]}

   store_data,'T_all',data='Tint2 Tint1 Text2 Text1 T-S T-B T-C',dlim={yrange:[-80,70],ystyle:1,ytitle:'Temperature (C)'}

;   tplot_names
  ; tplot,'I_Vreg V_R T_all I_Heater V_max V_atten V_mux'


   store_data,'AO0',data={x:dap.time,y:dap.data[16]}
   store_data,'AO1',data={x:dap.time,y:dap.data[17]}

end

pro load_TV_temps


   t = read_asc(/tags,/double,filter='*.prn')
   if keyword_set(t) then begin
   time  =t.time *3600d * 24d + time_double('2005-1-1')

   store_data,'T-C',data={x:time, y:[[t.T01],[t.T02],[t.T03]]},dlim={colors:'ygd'}
   store_data,'T-B',data={x:time,y:[[t.T08],[t.T10]]},dlim={colors:'cm'}
   store_data,'T-S',data={x:time,y:[[t.T09],[t.T11]]},dlim={colors:'br'}

   store_data,'Tse',data={x:time, y:[t.T01]},dlim={colors:'y'}
   store_data,'Tsm',data={x:time, y:[t.T02]},dlim={colors:'g'}
   store_data,'Tbp',data={x:time, y:[t.T03]},dlim={colors:'d'}
   store_data,'TB1',data={x:time, y:[t.T08]},dlim={colors:'b'}
   store_data,'TB2',data={x:time, y:[t.T10]},dlim={colors:'r'}
   store_data,'TS1',data={x:time, y:[t.T09]},dlim={colors:'b'}
   store_data,'TS2',data={x:time, y:[t.T11]},dlim={colors:'r'}

   endif
if 0 then begin
   tplot,'T_all I_Vreg I_Heater'

endif
end






;  begiin main

tplot_options,'ygap',.5

load_dap,dapfile
load_tv_temps
read_sstgse_dat,rawfile

end