;Read ESA APID 454 - Ion Survey 88 angle x 32 energies
;Assumes a fixed size for the packets
;Energy sweep determination requires definition

pro read_esagse_apid454,file=file,range=range,nsamples=nsamples,adat=adat,time=time,tshiftparam=tshiftparam,time_order=time_order


decompress=1.*[indgen(16),16+2*indgen(8),32+4*indgen(24),128+8*indgen(16),256+16*indgen(16),$
	512+32*indgen(16),1024+64*indgen(16),2048+128*indgen(48),8192+256*indgen(32),$
	16384+512*indgen(32),32768+1024*indgen(32)]

cols=get_colors()
if not keyword_set(file) then file = dialog_pickfile(filter='*_454.pkt')
openr, fp, file, /get_lun
fs = fstat(fp)
printdat,fs
if keyword_set(range) then begin
  ptr = round(double(range)*fs.size/2836) *2836
  point_lun,fp,ptr[0]
  nrec = (ptr[1]-ptr[0])/ 2836
endif else if keyword_set(nsamples) then begin
  ptr = fs.size-nsamples*2836l
  if ptr lt 0 then begin
    ptr=0
    nrec=fs.size/2836
  endif else nrec=nsamples
  point_lun,fp,ptr
endif else nrec = fs.size / 2836
dat = {	vers:bytarr(1),		$
	apid:bytarr(1), 	$
	sequ:bytarr(1), 	$
	cntr:bytarr(1), 	$
	leng:bytarr(2), 	$
	time:bytarr(6), 	$
	etc:bytarr(4), 		$
	config:bytarr(2), 	$
	spins:bytarr(2), 	$
	dat:bytarr(2816)}
adat = replicate(dat,nrec)


readu,fp,adat
free_lun,fp

tt=adat.time
time = 1.d*((tt(0,*)*256ul+tt(1,*))*256ul+tt(2,*))*256ul + tt(3,*) + (tt(4,*)+tt(5,*)/256.d)/256.
time = reform(time)
if keyword_set(time_order) then begin
	dat=adat.dat
	wset,1
	plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 454'					; test for time order
	wset,2
	cntr=reform(adat.cntr) & npkts=n_elements(cntr)
	plot,cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 454'
	oplot,10*(cntr(1:npkts-1)-cntr(0:npkts-2)-1) < 200,psym=0,color=cols.red
;	for i=0,nrec-1 do begin
;		ind=where(cntr(i) eq cntr,count)
;		if count gt 1 then begin
;			print,i,'  duplicate packet',ind
;			print,total(dat(*,*,ind(0))),'data differ=',total(dat(*,*,ind(0))-dat(*,*,ind(1)))
;		endif
;	endfor
	popen,/port,'454_packet_order'
		plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 454'					; test for time order
		plot,adat.cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 454'
	pclose
	print,'Exiting time order','  apid cntr',adat.cntr(0)
endif
time0 = time_double('1900-1-1')		
end_time=time+time0						; assume no delay for header creation
start_time=end_time-3.*(adat.spins(1)+1.)			; assume 3 sec spin period

if total(adat.spins(0)) gt total(adat.spins(1)) then print,'ERROR: read_esagse_apid454 - check spin byte

cdat=transpose(reform(adat.dat,88,32,nrec))
ddat=decompress(cdat)


;***********************************************************************************
;Energy-Angle mode determination 

emode=read_themis_esa_sweep_mode()

energy=emode.i_energy
denergy=emode.i_denergy
nenergy=emode.i_nenergy

en_ind=adat.etc(1)
en_ind(*)=0						; use default until headers correct
if max(en_ind) gt dimen2(energy) then begin
	print,'ERROR: 	Energy sweep table out of date'
	print,'		Download New Sweep Calibration Table'
	print,'		Energy sweeps out of range defaulting to Mode=0'
	ind=where(en_ind gt dimen2(energy)-1)
	en_ind(ind)=0
endif

amode=read_themis_esa_angle_mode()

theta=amode.i_theta
dtheta=amode.i_dtheta
phi=amode.i_phi
dphi=amode.i_dphi
domega=amode.i_domega
nbins=amode.i_nbins

an_ind=adat.etc(2)
an_ind(*)=1							; use test default until headers correct
if max(an_ind) gt dimen2(theta) then begin
	print,'ERROR: 	Angle mode table out of date'
	print,'		Download New Calibration Table'
	print,'		Angle modes out of range defaulting to Mode=0'
	ind=where(an_ind gt dimen2(theta)-1)
	an_ind(ind)=0
endif

;***********************************************************************************
; Make ion energy efficiency array 
; 	REF: Funsten curves for CEMs, assumes 2 kV pre-acceleration

eff=(1.+0.3*((energy+2000.)/30000.)^.5)/1.075


;***********************************************************************************
; Make 3D data structures

	anodes=theta & anodes(*)=1.
	ind=where(theta lt 22.5 and theta gt -22.5) & anodes(ind)=4.
	ind=where(theta lt 45. and theta gt 22.5) & anodes(ind)=2.
	ind=where(theta gt -45. and theta lt -22.5) & anodes(ind)=2.
	dt_arr=dphi/11.25*dtheta/22.5*anodes				; 32 sweeps / spin
	gf=dphi/22.5*dtheta/22.5
	geom_factor=.001						; geom_factor of 22.5x22.5 angle bin
	dt=end_time-start_time

i3d_dat = 	{project_name:		'THEMIS I&T',				$
		data_name:		'IESA 3D Survey', 			$
		data_product:		454,					$
		units_name: 		'compressed', 				$
		units_procedure: 	'convert_themis_units', 		$
		valid: 			1, 					$
		time: 			start_time,	 			$
		end_time: 		end_time, 				$
		delta_t: 		dt,					$
		integ_t: 		dt/1024.,				$
		dt_arr: 		dt_arr,					$
		dead: 			1.e-7,					$
		geom_factor: 		replicate(geom_factor,nrec),		$
		nenergy: 		nenergy, 				$
		nbins: 			nbins, 					$
		energy: 		energy, 				$
		denergy: 		denergy, 				$
		eff: 			eff,	 				$
		en_ind:			en_ind,					$
		an_ind:			an_ind,					$
		theta: 			theta,  				$
		dtheta: 		dtheta,  				$
		phi: 			phi,  					$
		dphi: 			dphi,  					$
		domega: 		domega,  				$
		gf: 			gf, 					$
		mass: 			1.0438871e-02, 				$
		charge: 		1., 					$
		sc_pot: 		replicate(0.,nrec), 			$
		data: 			cdat}

common th_i3d_454,get_ind,all_dat & all_dat=i3d_dat & get_ind=0

help,all_dat,/st

;***********************************************************************************
; Make tplot structures

en_arr=fltarr(nrec,32)
;en_ind(100:150)=1					; test purposes
for i=0,nrec-1 do en_arr(i,*)=energy(*,en_ind(i))

store_data,'i+_ien_454',data={x:start_time,y:reform(total(1.*ddat,3)),v:replicate(1.,nrec)#findgen(32)}
	ylim,'i+_ien_454',-1.,32.,0
	options,'i+_ien_454','spec',1
	zlim,'i+_ien_454',0,0,1
	options,'i+_ien_454','x_no_interp',1
	options,'i+_ien_454','y_no_interp',1
en16=where(nenergy(en_ind) eq 16,count)
if count eq 0 then store_data,'i+_en_454',data={x:start_time,y:reform(total(1.*ddat,3)),v:en_arr}
if count ne 0 then begin
	new_time=dblarr(nrec+count)
	new_dat=fltarr(nrec+count,32)
	new_en_arr=fltarr(nrec+count,32)
	nn=0
	for i=0,nrec-1 do begin
		if nenergy(en_ind(i)) eq 32 then begin
			new_time(nn)=start_time(i)
			new_dat(nn,*)=reform(total(1.*ddat(i,*,*),3))
			new_en_arr(nn,*)=en_arr(i,*)
			nn=nn+1
		endif else begin
			new_time(nn)=start_time(i)
			new_dat(nn,0:15)=reform(total(1.*ddat(i,0:15,*),3))
			new_en_arr(nn,0:15)=en_arr(i,0:15)
			new_en_arr(nn,16:31)=en_arr(i,15)
			nn=nn+1
			new_time(nn)=start_time(i)+1.5				; assume 3 second spin period
			new_dat(nn,0:15)=reform(total(1.*ddat(i,16:31,*),3))
			new_en_arr(nn,0:15)=en_arr(i,16:31)
			new_en_arr(nn,16:31)=en_arr(i,31)
			nn=nn+1
		endelse
	endfor
	store_data,'i+_en_454',data={x:new_time,y:new_dat,v:new_en_arr}
endif
	zlim,'i+_en_454',0,0,1
	ylim,'i+_en_454',1.,50000.,1
	options,'i+_en_454','spec',1
	options,'i+_en_454','x_no_interp',1
	options,'i+_en_454','y_no_interp',1
store_data,'i+_ian_454',data={x:start_time,y:reform(total(1.*ddat,2)),v:findgen(88)}
	zlim,'i+_ian_454',0,0,0
	options,'i+_ian_454','spec',1
	options,'i+_ian_454','x_no_interp',1
	options,'i+_ian_454','y_no_interp',1

store_data,'time_454',data={x:start_time,y:start_time-start_time(0)}
	options,'time_454','psym',1
get_2dt,'c_3d','th_454',name='Counts_454'
	ylim,'Counts_454',.1,1.e6,1
get_en_spec,'th_454',name='Spec_454'
	zlim,'Spec_454',0,0,1
	ylim,'Spec_454',1.,50000.,1
	options,'Spec_454','spec',1
	options,'Spec_454','x_no_interp',1
	options,'Spec_454','y_no_interp',1

tplot,['i+_ien_454','i+_en_454','i+_ian_454','Spec_454','Counts_454','time_454']

;***********************************************************************************

end
