;Read ESA APID 455 - Ion Survey 6 angle x 16 energies
;Assumes a fixed size for the packets
;Energy sweep determination requires definition

pro read_esagse_apid455,file=file,range=range,nsamples=nsamples,adat=adat,time=time,tshiftparam=tshiftparam,time_order=time_order

decompress=1.*[indgen(16),16+2*indgen(8),32+4*indgen(24),128+8*indgen(16),256+16*indgen(16),$
	512+32*indgen(16),1024+64*indgen(16),2048+128*indgen(48),8192+256*indgen(32),$
	16384+512*indgen(32),32768+1024*indgen(32)]

cols=get_colors()
if not keyword_set(file) then file = dialog_pickfile(filter='*_455.pkt')
openr, fp, file, /get_lun
fs = fstat(fp)
printdat,fs
if keyword_set(range) then begin
  ptr = round(double(range)*fs.size/3216) *3216
  point_lun,fp,ptr[0]
  nrec = (ptr[1]-ptr[0])/ 3216
endif else if keyword_set(nsamples) then begin
  ptr = fs.size-nsamples*3216l
  if ptr lt 0 then begin
    ptr=0
    nrec=fs.size/3216
  endif else nrec=nsamples
  point_lun,fp,ptr
endif else nrec = fs.size / 3216
dat = {	vers:bytarr(1),		$
	apid:bytarr(1), 	$
	sequ:bytarr(1), 	$
	cntr:bytarr(1), 	$
	leng:bytarr(2), 	$
	time:bytarr(6), 	$
	etc:bytarr(4), 		$
	dat:bytarr(100,32)}
adat = replicate(dat,nrec)


readu,fp,adat
free_lun,fp

tt=adat.time
time = 1.d*((tt(0,*)*256ul+tt(1,*))*256ul+tt(2,*))*256ul + tt(3,*) + (tt(4,*)+tt(5,*)/256.d)/256.
time = reform(time)
if keyword_set(time_order) then begin
	dat=adat.dat
	wset,1
	plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 455'					; test for time order
	wset,2
	cntr=reform(adat.cntr) & npkts=n_elements(cntr)
	plot,cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 455'
	oplot,10*(cntr(1:npkts-1)-cntr(0:npkts-2)-1) < 200,psym=0,color=cols.red
;	for i=0,nrec-1 do begin
;		ind=where(cntr(i) eq cntr,count)
;		if count gt 1 then begin
;			print,i,'  duplicate packet',ind
;			print,total(dat(*,*,ind(0))),'data differ=',total(dat(*,*,ind(0))-dat(*,*,ind(1)))
;		endif
;	endfor
	popen,/port,'455_packet_order'
		plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 455'					; test for time order
		plot,adat.cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 455'
	pclose
	print,'Exiting time order'
endif
time0 = time_double('1900-1-1')		
tmp_time=time+time0						; assume no delay for header creation
end_time=dblarr(32l*nrec)
start_time=dblarr(32l*nrec)

tmp=adat.dat
config=tmp(0:1,*,*)
spin=reform(tmp(3,*,*))+1
spins=intarr(32)
cdat=transpose(reform(tmp(4:99,*,*),6,16,32*nrec))
ddat=1.*decompress(cdat)

for i=0,nrec-1 do begin
	for j=0,31 do spins(j)=total(spin(j:31,i))
	start_time(i*32:i*32+31)=tmp_time(i)-spins*3.		; assume 3 sec spin
	end_time(i*32:i*32+30)=start_time(1+i*32:i*32+31)	; assume no gaps in average
	end_time(i*32+31)=tmp_time(i)
endfor	
	
;***********************************************************************************
;Energy-Angle mode determination 

emode=read_themis_esa_sweep_mode()

energy=emode.i_energy
denergy=emode.i_denergy
nenergy=16

if dimen1(energy) eq 32 then begin
	energy=(energy(indgen(16)*2)+energy(indgen(16)*2+1))/2.
	denergy=denergy(indgen(16)*2)+denergy(indgen(16)*2+1)
endif else begin
	print,'ERROR: energy dimension problem'
	energy=energy(0:15)
	denergy=denergy(0:15)
endelse

en_ind=adat.etc(1)					; assume this header byte defines energy mode
en_ind(*)=0						; use default until headers correct
en_ind=reform(replicate(1.,32)#en_ind,32l*nrec)		; assume energy sweep constant during packet

if max(en_ind) gt dimen2(energy) then begin
	print,'ERROR: 	Energy sweep table out of date'
	print,'		Download New Sweep Calibration Table'
	print,'		Energy sweeps out of range defaulting to Mode=0'
	ind=where(en_ind gt dimen2(energy)-1)
	en_ind(ind)=0
endif

; assume the solid angles never change for this packet

theta=replicate(1.,16)#[67.5,-67.5,0.,0.,0.,0.]
dtheta=replicate(1.,16)#[45.,45.,90.,90.,90.,90.]
phi=replicate(1.,16)#[0.,0.,45.,135.,225.,315.]
dphi=replicate(1.,16)#[360.,360.,90.,90.,90.,90.]
	p0=theta+dtheta/2.
	p1=theta-dtheta/2.
domega=2*!pi*dphi/360.*(sin(p0/!radeg)-sin(p1/!radeg))
nbins=6

;***********************************************************************************
; Make ion energy efficiency array 
; 	REF: Funsten curves for CEMs, assumes 2 kV pre-acceleration

eff=(1.+0.3*((energy+2000.)/30000.)^.5)/1.075

;***********************************************************************************
; Make 3D data structures

	anodes=theta & anodes(*)=12.
	ind=where(theta gt 45. or theta lt -45.) & anodes(ind)=2.
	dt_arr=dphi/11.25*anodes
	gf=dphi/22.5*dtheta/22.5
	geom_factor=.001						; geom_factor of 22.5x22.5 angle bin
	dt=end_time-start_time

i3d_dat = 	{project_name:		'THEMIS I&T',				$
		data_name:		'IESA 3D Reduced', 			$
		data_product:		455,					$
		units_name: 		'compressed', 				$
		units_procedure: 	'convert_themis_units', 		$
		valid: 			1, 					$
		time: 			start_time,	 			$
		end_time: 		end_time, 				$
		delta_t: 		dt,					$
		integ_t: 		dt/1024.,				$
		dt_arr: 		dt_arr,					$
		dead: 			1.e-7,					$
		geom_factor: 		replicate(geom_factor,32l*nrec),	$
		nenergy: 		16, 					$
		nbins: 			6, 					$
		energy: 		energy, 				$
		denergy: 		denergy, 				$
		eff: 			eff,	 				$
		en_ind:			en_ind,					$
		theta: 			theta,  				$
		dtheta: 		dtheta,  				$
		phi: 			phi,  					$
		dphi: 			dphi,  					$
		domega: 		domega,  				$
		gf: 			gf, 					$
		mass: 			1.0438871e-02, 				$
		charge: 		1., 					$
		sc_pot: 		replicate(0.,32l*nrec), 		$
		data: 			cdat}

common th_i3d_455,get_ind,all_dat & all_dat=i3d_dat & get_ind=0

help,all_dat,/st

;***********************************************************************************
; Make tplot structures

en_arr=fltarr(nrec,16)
;en_ind(100:150)=1					; test purposes
for i=0,nrec-1 do en_arr(i,*)=energy(*,en_ind(i))

store_data,'i+_ien_455',data={x:start_time,y:reform(total(ddat,3)),v:replicate(1.,nrec)#findgen(16)}
	ylim,'i+_ien_455',-1.,16.,0
	zlim,'i+_ien_455',0,0,1
	options,'i+_ien_455','spec',1
	options,'i+_ien_455','x_no_interp',1
	options,'i+_ien_455','y_no_interp',1

store_data,'i+_en_455',data={x:start_time,y:reform(total(ddat,3)),v:en_arr}
	zlim,'i+_en_455',0,0,1
	ylim,'i+_en_455',1.,50000.,1
	options,'i+_en_455','spec',1
	options,'i+_en_455','x_no_interp',1
	options,'i+_en_455','y_no_interp',1

store_data,'i+_ian_455',data={x:start_time,y:reform(total(ddat,2)),v:findgen(6)}
	zlim,'i+_ian_455',0,0,1
	options,'i+_ian_455','spec',1
	options,'i+_ian_455','x_no_interp',1
	options,'i+_ian_455','y_no_interp',1

store_data,'time_455',data={x:start_time,y:start_time-start_time(0)}
	options,'time_455','psym',1
get_2dt,'c_3d','th_455',name='Counts_455'
	ylim,'Counts_455',.1,1.e6,1
get_en_spec,'th_455',name='Spec_455'
	zlim,'Spec_455',0,0,1
	ylim,'Spec_455',1.,50000.,1
	options,'Spec_455','spec',1
	options,'Spec_455','x_no_interp',1
	options,'Spec_455','y_no_interp',1

tplot,['i+_ien_455','i+_en_455','i+_ian_455','Spec_455','Counts_455','time_455']

;***********************************************************************************

end
