;Read ESA APID 459 - Electron Burst 88 angle x 32 energies
;Assumes a fixed size for the packets
;Energy sweep determination requires definition

pro read_esagse_apid459,file=file,range=range,nsamples=nsamples,adat=adat,time=time,tshiftparam=tshiftparam,time_order=time_order


decompress=1.*[indgen(16),16+2*indgen(8),32+4*indgen(24),128+8*indgen(16),256+16*indgen(16),$
	512+32*indgen(16),1024+64*indgen(16),2048+128*indgen(48),8192+256*indgen(32),$
	16384+512*indgen(32),32768+1024*indgen(32)]

cols=get_colors()
if not keyword_set(file) then file = dialog_pickfile(filter='*_459.pkt')
openr, fp, file, /get_lun
fs = fstat(fp)
printdat,fs
if keyword_set(range) then begin
  ptr = round(double(range)*fs.size/2836) *2836
  point_lun,fp,ptr[0]
  nrec = (ptr[1]-ptr[0])/ 2836
endif else if keyword_set(nsamples) then begin
  ptr = fs.size-nsamples*2836l
  if ptr lt 0 then begin
    ptr=0
    nrec=fs.size/2836
  endif else nrec=nsamples
  point_lun,fp,ptr
endif else nrec = fs.size / 2836
dat = {	vers:bytarr(1),		$
	apid:bytarr(1), 	$
	sequ:bytarr(1), 	$
	cntr:bytarr(1), 	$
	leng:bytarr(2), 	$
	time:bytarr(6), 	$
	etc:bytarr(4), 		$
	config:bytarr(2), 	$
	spins:bytarr(2), 	$
	dat:bytarr(2816)}
adat = replicate(dat,nrec)


readu,fp,adat
free_lun,fp

tt=adat.time
time = 1.d*((tt(0,*)*256ul+tt(1,*))*256ul+tt(2,*))*256ul + tt(3,*) + (tt(4,*)+tt(5,*)/256.d)/256.
time = reform(time)
if keyword_set(time_order) then begin
	dat=adat.dat
	wset,1
	plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 459'					; test for time order
	wset,2
	cntr=reform(adat.cntr) & npkts=n_elements(cntr)
	plot,cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 459'
	oplot,10*(cntr(1:npkts-1)-cntr(0:npkts-2)-1) < 200,psym=0,color=cols.red
;	for i=0,nrec-1 do begin
;		ind=where(cntr(i) eq cntr,count)
;		if count gt 1 then begin
;			print,i,'  duplicate packet',ind
;			print,total(dat(*,*,ind(0))),'data differ=',total(dat(*,*,ind(0))-dat(*,*,ind(1)))
;		endif
;	endfor
	popen,/port,'459_packet_order'
		plot,time-time(0),psym=1,ytitle='Seconds after first packet',xtitle='Packet Number',title='APID 459'					; test for time order
		plot,adat.cntr,psym=1,ytitle='APID Counter',xtitle='Packet Number',title='APID 459'
	pclose
	print,'Exiting time order'
endif
time0 = time_double('1900-1-1')		
end_time=time+time0						; assume no delay for header creation
start_time=end_time-3.*(adat.spins(1)+1.)			; assume 3 sec spin period

if total(adat.spins(0)) gt total(adat.spins(1)) then print,'ERROR: read_esagse_apid459 - check spin byte

cdat=transpose(reform(adat.dat,88,32,nrec))
ddat=decompress(cdat)


;***********************************************************************************
;Energy-Angle mode determination 

emode=read_themis_esa_sweep_mode()

energy=emode.e_energy
denergy=emode.e_denergy
nenergy=emode.e_nenergy

en_ind=adat.etc(1)
en_ind(*)=0						; use default until headers correct
if max(en_ind) gt dimen2(energy) then begin
	print,'ERROR: 	Energy sweep table out of date'
	print,'		Download New Sweep Calibration Table'
	print,'		Energy sweeps out of range defaulting to Mode=0'
	ind=where(en_ind gt dimen2(energy)-1)
	en_ind(ind)=0
endif

amode=read_themis_esa_angle_mode()

theta=amode.e_theta
dtheta=amode.e_dtheta
phi=amode.e_phi
dphi=amode.e_dphi
domega=amode.e_domega
nbins=amode.e_nbins

an_ind=adat.etc(2)
an_ind(*)=0							; use default until headers correct
if max(an_ind) gt dimen2(theta) then begin
	print,'ERROR: 	Angle mode table out of date'
	print,'		Download New Calibration Table'
	print,'		Angle modes out of range defaulting to Mode=0'
	ind=where(an_ind gt dimen2(theta)-1)
	an_ind(ind)=0
endif

;***********************************************************************************
; Make electron energy efficiency array 
; 	REF: Relative electron detection efficiency of microchannel plates from 0-3 keV
; 	R.R. Goruganthu and W. G. Wilson, Rev. Sci. Instrum. Vol. 55, No. 12 Dec 1984

	tmax = 2.283
	aa = 1.35
	delta_max = 1.0
	emax = 325.  ;eV
	en_acc = 600. ;eV for THEMIS
	delta = delta_max * ((energy+en_acc)/emax)^(1-aa)*(1-exp(-tmax*((energy+en_acc)/emax)^aa))/(1-exp(-tmax))
	k = 2.2
	eff = (1-exp(-k*delta/delta_max))/(1-exp(-k))

;***********************************************************************************
; Make 3D data structures

	dt_arr=dphi/11.25*dtheta/22.5					; 32 sweeps / spin
	gf=dphi/22.5*dtheta/22.5
	geom_factor=.001						; geom_factor of 22.5x22.5 angle bin
	dt=end_time-start_time


e3d_dat = 	{project_name:		'THEMIS I&T',				$
		data_name:		'EESA 3D Burst', 			$
		data_product:		459,					$
		units_name: 		'compressed', 				$
		units_procedure: 	'convert_themis_units', 		$
		valid: 			1, 					$
		time: 			start_time,	 			$
		end_time: 		end_time, 				$
		delta_t: 		dt,					$
		integ_t: 		dt/1024.,				$
		dt_arr: 		dt_arr,					$
		dead: 			1.e-7,					$
		geom_factor: 		replicate(geom_factor,nrec),		$
		nenergy: 		nenergy, 				$
		nbins: 			nbins, 					$
		energy: 		energy, 				$
		denergy: 		denergy, 				$
		eff: 			eff,	 				$
		en_ind:			en_ind,					$
		an_ind:			an_ind,					$
		theta: 			theta,  				$
		dtheta: 		dtheta,  				$
		phi: 			phi,  					$
		dphi: 			dphi,  					$
		domega: 		domega,  				$
		gf: 			gf, 					$
		mass: 			5.68566e-06, 				$
		charge: 		-1., 					$
		sc_pot: 		replicate(0.,nrec), 			$
		data: 			cdat}

common th_e3d_459,get_ind,all_dat & all_dat=e3d_dat & get_ind=0

help,all_dat

;***********************************************************************************
; Make tplot structures

en_arr=fltarr(nrec,32)
for i=0,nrec-1 do en_arr(i,*)=energy(*,en_ind(i))

store_data,'e-_ien_459',data={x:start_time,y:reform(total(ddat,3)),v:replicate(1.,nrec)#findgen(32)}
	zlim,'e-_ien_459',0,0,1
	ylim,'e-_ien_459',-1.,32.,0
	options,'e-_ien_459','spec',1
	options,'e-_ien_459','x_no_interp',1
	options,'e-_ien_459','y_no_interp',1
store_data,'e-_en_459',data={x:start_time,y:reform(total(ddat,3)),v:en_arr}
	zlim,'e-_en_459',0,0,1
	ylim,'e-_en_459',1.,50000.,1
	options,'e-_en_459','spec',1
	options,'e-_en_459','x_no_interp',1
	options,'e-_en_459','y_no_interp',1
store_data,'e-_ian_459',data={x:start_time,y:reform(total(ddat,2)),v:findgen(88)}
	zlim,'e-_ian_459',0,0,1
	options,'e-_ian_459','spec',1
	options,'e-_ian_459','x_no_interp',1
	options,'e-_ian_459','y_no_interp',1

store_data,'time_459',data={x:start_time,y:start_time-start_time(0)}
	options,'time_459','psym',1
get_2dt,'c_3d','th_459',name='Counts_459'
	ylim,'Counts_459',.1,1.e6,1
get_en_spec,'th_459',name='Spec_459'
	zlim,'Spec_459',0,0,1
	ylim,'Spec_459',1.,50000.,1
	options,'Spec_459','spec',1
	options,'Spec_459','x_no_interp',1
	options,'Spec_459','y_no_interp',1

tplot,['e-_ien_459','e-_en_459','e-_ian_459','Spec_459','Counts_459','time_459']

;***********************************************************************************

end
