
function teta_order, tzo, torder
; by default: 1A, 2B, 1B, 2A

 tzz  = replicate(0.,16,64)
 bnrj = lindgen(16)

 for tt=0,3 do begin         ; loop over 4 look-directions
   bin = torder[tt]+bnrj*4
   bic = tt+bnrj*4
   tzz[*,bic] = tzo[*,bin]
 endfor

 return, tzz
end




pro movie_plot, d,w,c, torder, lim=lim

 for wn=0,c-1l do begin
  z = d.y(w[wn],*)
  t = [d.x(w[wn]),d.x(w[wn]+1)]
  str_element,lim,'title',d.name+'  '+trange_str(t),/add_replace
  plotspec, z, torder, lim=lim, flag_mov=1
  wait,  .3 
 endfor

end






pro plotspec, z, torder, lim=lim, flag_mov=flag_mov

  z = average(z,1,/nan)     ; count / Accumulation time 
;  z = total(z,1,/nan)

  zz = replicate(0.,16,64)
  xx = lindgen(16)
  yy = lindgen(64)
  bin = replicate(0,64)

  for i=0,15 do begin        ; loop over the 16 spin sectors 
    bin  = 16*yy+i
    zz[i,yy] = z[bin]
  endfor

  zz = teta_order(zz,torder)  

  if not keyword_set(flag_mov) then $
  str_element,lim,'zrange',[.8,max([z,1.])],/add
  str_element,lim,'no_interp',1,/add
  ;  str_element,lim,'zrange',[.8,200.],/add
  tspec,xx,yy,zz,no_interp=1,lim=lim

end










;+
;PROCEDURE:  cutspec [,t]
;PURPOSE:
;   Creates a 16 spin sectors x 16 energy bands x 4 apertures spectra of 
;   the "spec" full distribution [1024bytes x time] in a time range selected.
;KEYWORDS:
;   np:  1 for a time shot  -  2 for a time range
;   ncuts: Number of time cuts (by default it's infinite)
;   movie: Show continuous 3s spectras in the time range selected
;   torder: TO display the group stack order (by default 1A, 1B, 2A, 2B)
;-
pro cutspec,np=np,t,ncuts=ncuts,movie=movie, torder=torder
;used with 

@tplot_com.pro

; setup tplot_vars....
title_old=tplot_vars.options.title

if n_elements(torder) eq 0 then torder=[0,2,3,1]      ; 1A, 1B, 2A, 2B
if n_elements(np) eq 0 then np=2
if n_elements(ncuts) eq 0 then ncuts=10000

for i=0,ncuts-1  do begin
 ctime,t,np=np,vnam=varname,noshow=i
 varnames=varname
 n = 0
 repeat begin
  if not keyword_set(t) then return
  get_data,varnames[0],data=vn
  d={x:vn.x, y:vn.y, v:vn.v, name:varnames[0]}

  lim = 0
  str_element,lim,'title',varnames[0]+'  '+trange_str(t),/add
  str_element,lim,'zlog',1,/add
  str_element,lim,'zticks',1,/add
  str_element,lim,'ztitle','Count',/add
  str_element,lim,'xtitle','Phi Angle',/add
  str_element,lim,'ytitle','4 look direction  x  16 bin energy',/add

  ;Create a Y-axis with a title:
  !y.tickname=' '
  str_element,/add,lim,'yticklen',1
  str_element,/add,lim,'yticks',16
  str_element,/add,lim,'ytickv',[4*lindgen(16)]

  ytags=replicate(' ',17)
  ytags[0:15]=Strcompress(lindgen(16),/REMOVE_ALL)
  str_element,/add,lim,'ytickname',ytags


  if not keyword_set(d) then return

  if n_elements(t) eq 2 then w = where(d.x ge t(0) and d.x lt t(1),c)
  if n_elements(t) eq 1 then dummy = min(abs(d.x - t[0]),w)

  print, 'max',max(d.y(w,*))
  str_element,lim,'zrange',[0.8,max([max(d.y(w,*)),1.])],/add

  if keyword_set(movie) then  $
         movie_plot, d,w,c, torder, lim=lim   $
  else   plotspec, d.y(w,*), torder, lim=lim

  str_element,tplot_vars,'options.title',title_old,/add_replace
  !y.tickname=''

 endrep until 1
endfor

end

