;+
;PROCEDURE:	tspec, xdata, ydata, zdata
; Called by cutspec
;PURPOSE:
;   Produces a (M,N) spectrogram of selected tplot spectrograms.
;   Colors are scaled according to zname
;KEYWORDS::
;   xdata array of M elements
;   ydata array of N elements
;   zdata array of (M,N) elements
;-

pro tspec, xdata,ydata,zdata,      $
   WINDOW = wind,                  $
   OVERPLOT = overplot,            $
   OVERLAY = overlay,              $
   LIMITS = lim,                   $
   PS_RESOLUTION = ps_res,         $
   X_NO_INTERP = x_no_interp,      $
   Y_NO_INTERP = y_no_interp,      $
   NO_INTERP = no_interp

@tplot_com.pro



chsize = !p.charsize
if chsize eq 0. then chsize=1.
             
                                                                                
opt = {xrange:[0.,0.],yrange:[0.,0.],zrange:[1.,1.],     $
   ymargin:[4.,2.],xmargin:[12.,12.],position:fltarr(4), $
   title:'',xtitle:'',ytitle:'',ztitle:'',               $
   xstyle:1,ystyle:1,zstyle:1,charsize:chsize,           $
   window:-1,wshow:0,noerase:0,                          $
   overplot:0,overlay:0,no_color_scale:0,                $
   x_no_interp:0,y_no_interp:0,no_interp:0}


; setup tplot_vars....
tplot_options,wind=wind
extract_tags,opt,tplot_vars.options
extract_tags,opt,limits


if (!d.flags and 256) ne 0  then begin    ; windowing devices
   current_window= !d.window > 0
   if opt.window gt 0 then w = opt.window $
   else w = current_window
   if !d.window eq 0 and opt.window eq 0  then begin
      w = 1
      window, w
   endif
   wset,w
   if opt.wshow ne 0 then wshow,icon=0
endif


extract_tags,opt,lim

str_element,opt,'xrange',value=xrange
str_element,opt,'yrange',value=yrange

if opt.xrange(0) eq opt.xrange(1) then  opt.xrange = minmax(xdata)
if opt.yrange(0) eq opt.yrange(1) then  opt.yrange = minmax(ydata)



if opt.no_interp eq 1 then begin
   x_no_interp=1
   y_no_interp=1
endif


if not keyword_set(overplot) then  box, opt

str_element,opt,'zlog',value=zlog

str_element,opt,'max_value',value=mx
str_element,opt,'min_value',value=mn

zrange = opt.zrange

if keyword_set(zlog) then begin
   bad = where( finite(zdata) eq 0, c)
   if c ne 0 then zdata(bad) = 0.
   bad = where(zdata le 0,c)
   if c ne 0 then zdata(bad) = !values.f_nan
   zdata = alog10(zdata)
   zrange = alog10(zrange)
   if keyword_set(mn) then mn = alog10(mn)
   if keyword_set(mx) then mx = alog10(mx)
endif



xwindow=!x.window
ywindow=!y.window
xcrange=!x.crange
ycrange=!y.crange


str_element,opt,'overlay',value=overlay,/add_replace
if keyword_set(overlay) then begin
   winpos = convert_coord(minmax(xdata),minmax(ydata),/data,/to_norm)
   xwr = minmax(winpos(0,*))
   ywr = minmax(winpos(1,*))
   xwindow = xwindow > xwr[0]
   xwindow = xwindow < xwr[1]
   ywindow(0) = ywindow(0) > ywr(0)
   ywindow(1) = ywindow(1) < ywr(1)
   datpos = convert_coord(xwindow,ywindow,/norm,/to_data)
   xcrange = reform(datpos(0,*))
   ycrange = reform(datpos(1,*))
   if !x.type then xcrange = alog10(xcrange)
   if !y.type then ycrange = alog10(ycrange)
endif


pixpos = round(convert_coord(xwindow,ywindow,/norm,/to_device))
npx = pixpos(0,1)-pixpos(0,0)+1
npy = pixpos(1,1)-pixpos(1,0)+1
xposition = pixpos(0,0)
yposition = pixpos(1,0)

if npx gt 0 and npy gt 0 then begin

if !d.flags and 1 then begin   ; scalable pixels (postscript)
   if keyword_set(ps_res) then ps_resolution=ps_res else  ps_resolution = 60.  ; Postscript defaults to 60 dpi
   str_element,opt,'ps_resolution',value=ps_resolution
   scale = ps_resolution/!d.x_px_cm/2.54
   print, scale
endif else scale = 1.


nypix = round(scale*npy)
ny = n_elements(ydata)
yp = findgen(nypix)*(ycrange(1)-ycrange(0))/(nypix-1) + ycrange(0)
ys = interp(findgen(ny),ydata,yp)
if keyword_set(y_no_interp) then  ys = round(ys)

nxpix = round(scale*npx)
nx = n_elements(xdata)
xp = findgen(nxpix)*(xcrange(1)-xcrange(0))/(nxpix-1) + xcrange(0)
xs = interp(findgen(nx),xdata,xp )
if keyword_set(x_no_interp) then  xs = round(xs)

image = interpolate(zdata,xs,ys,missing = !values.f_nan,/grid)

str_element,opt,'ztitle',value=ztitle
str_element,opt,'bottom',value=bottom
str_element,opt,'top',   value=top


if not keyword_set(gifplot) then begin

  if zrange(0) eq zrange(1) then $
     zrange = minmax(image,max=mx,min=mn)
  image = bytescale(image,bottom=bottom,top=top,range=zrange)
endif


tv,image,xposition,yposition,xsize=npx,ysize=npy

;redraw the axes
str_element,/add,opt,'noerase',1
str_element,/add,opt,'overplot',/delete
str_element,/add,opt,'ytitle',/delete
str_element,/add,opt,'position',reform(transpose([[!x.window],[!y.window]]),4)
box,opt

endif else stop



if keyword_set(zlog) then zrange = 10.^zrange

charsize=!p.charsize
str_element,opt,'charsize',value=charsize
if not keyword_set(charsize) then charsize = 1.

str_element,opt,'no_color_scale',value=no_color_scale
str_element,opt,'zticks',zticks
str_element,opt,'zposition',zposition
str_element,opt,'zoffset',zoffset
!y.tickname=''
if not keyword_set(no_color_scale) then $
  draw_color_scale,brange=[bottom,top],range=zrange,log=zlog,title=ztitle, $
    charsize=charsize,yticks=zticks,position=zposition,offset=zoffset

end
