

;+
;FUNCTION:  IGAUSS
;PURPOSE:
;  Evaluates integral of gaussian function .
;  This function may be used with the "fit" curve fitting procedure.
;
;KEYWORDS:
;  PARAMETERS: structure with the format:
;** Structure <275ac0>, 6 tags, length=48, refs=1:
;   A               DOUBLE           1.0000000      ; Area
;   S               DOUBLE           1.0000000      ; width (sigma)
;   X0              DOUBLE           0.0000000      ; center
;     If this parameter is not a structure then it will be created.
;
;USAGE:
;  p=igauss()
;  x = indgen(100)
;  dx = x*0+1
;  y = igauss(x,dx)
;  plot,x,y
;RETURNS:
;
;-

function igauss, x,dx, parameters=p


if not keyword_set(p) then $
   p = {func:"igauss",A:1.d, x0:0.d, s:1.0d }

if n_params() eq 0 then return,p
n = n_elements(x)
if n_elements(dx) eq 0 then begin
   dx = shift(x,-1)-x
   dx[n-1]=dx[n-2]
endif
x1=x
x2=x+dx
s2 = sqrt(2)
z1 = (x1-p.x0)/p.s/s2
z2 = (x2-p.x0)/p.s/s2
f = p.a/2 * (erfc(z1) - erfc(z2))

return,f
end


