
pro load_dap,dapfiles,data=dap

if  not keyword_set(dapfiles) then begin
   dapfiles = dialog_pickfile(filt='*.dap',/multi)
   if n_elements(dapfiles) gt 1 then    dapfiles = dapfiles(sort(dapfiles))
   if not keyword_set(dapfiles) then return
endif

if not keyword_set(dap) then begin
   dap = read_daq(dapfiles[0])
   if not keyword_set(dap) then return
   for i =1,n_elements(dapfiles)-1 do   dap =  [dap,read_daq(dapfiles[i])]
   dt = dap.time -shift(dap.time,1)
   w = where(dt gt 4,nw)
   if nw gt 1 then dap[w].data = !values.f_nan
endif

;timespan,minmax(dap.time)

   ;printdat,dap
   store_data,'I_Preg1',data={x:dap.time,y:  dap.data[0]/.0031}, dlim={colors:'b'}
   store_data,'I_Preg2',data={x:dap.time,y:  dap.data[1]/.0031}, dlim={colors:'r'}
   store_data,'I_Nreg1',data={x:dap.time,y: -dap.data[2]/.0062}, dlim={colors:'b'}
   store_data,'I_Nreg2',data={x:dap.time,y: -dap.data[3]/.0062}, dlim={colors:'r'}
   store_data,'I_Vreg',data='I_Preg2 I_Preg1 I_Nreg2 I_Nreg1',dlim={yrange:[0,30.],ystyle:1,ytitle:'Sensor Current (mA)'}

   V_bias = dap.data[10]
   V_rint1 = dap.data[4]
   V_rint2 = dap.data[5]
   V_rext1 = dap.data[8]
   V_rext2 = dap.data[9]

   w = where(v_bias lt 3.8,nw)
   if nw gt 0 then v_bias[w] = !values.f_nan

   dv = V_bias-4.0
   V_bias = V_bias - dv
   V_rint1= V_rint1 + .7*dv
   V_rint2= V_rint2 + .7*dv
   V_rext1= V_rext1 - dv
   V_rext2= V_rext2 - dv

   w = where(V_rext1 lt .1,nw)
   if nw gt 0 then V_rext1[w] = !values.f_nan
   w = where(V_rext2 lt .1,nw)
   if nw gt 0 then V_rext2[w] = !values.f_nan
   w = where(V_rint1 lt .1,nw)
   if nw gt 0 then V_rint1[w] = !values.f_nan
   w = where(V_rint2 lt .1,nw)
   if nw gt 0 then V_rint2[w] = !values.f_nan

   store_data,'V_Rext1',data={x:dap.time,y: V_rext1}, dlim={colors:'b'}
   store_data,'V_Rext2',data={x:dap.time,y: V_rext2}, dlim={colors:'r'}
   store_data,'V_Rext',data='V_Rext1 V_Rext2'
   store_data,'V_Rint1',data={x:dap.time,y: V_rint1}, dlim={colors:'b'}
   store_data,'V_Rint2',data={x:dap.time,y: V_rint2}, dlim={colors:'r'}
   store_data,'V_R',data='V_Rint1 V_Rint2 V_Rext1 V_Rext2 V_atten',dlim={yrange:[0,5],ystyle:1}


   if not keyword_set(pti1) then begin
      pti1 = therm_temp2()
      pti1.r1=10000.
      pti1.xmax=2.431
   endif
   if not keyword_set(pti2) then begin
      pti2 = therm_temp2()
      pti2.r1=10000.
      pti2.xmax=2.444
   endif

   Tint1 = therm_temp2(V_rint1,param=pti1)
   Tint2 = therm_temp2(V_rint2,param=pti2)
   store_data,'Tint1',data={x:dap.time,y: Tint1}, dlim={colors:'b'}
   store_data,'Tint2',data={x:dap.time,y: Tint2}, dlim={colors:'r'}
   store_data,'Tint',data='Tint1 Tint2'



   store_data,'I_Heater1',data={x:dap.time,y: dap.data[6]}, dlim={colors:'b',yrange:[-.05,.35],ystyle:1}
   store_data,'I_Heater2',data={x:dap.time,y: dap.data[7]}, dlim={colors:'r',yrange:[-.05,.35],ystyle:1}
   store_data,'I_Heater',data='I_Heater2 I_Heater1'        ,dlim={yrange:[-.05,.35],ystyle:1,panel_size:.3}


   pt1 = therm_temp2()  & pt1.r1=50000.   & pt1.xmax=1.0
   pt2 = pt1
   Text1 = therm_temp2(V_rext1/V_bias,param=pt1)
   Text2 = therm_temp2(V_rext2/v_bias,param=pt2)
   store_data,'Text1',data={x:dap.time,y: Text1}, dlim={colors:'b'}
   store_data,'Text2',data={x:dap.time,y: Text2}, dlim={colors:'r'}
   store_data,'Text',data='Text1 Text2'

   store_data,'V_atten',data={x:dap.time,y: dap.data[10]}
   store_data,'V_mux',  data={x:dap.time,y: dap.data[11]}

   store_data,'T_all',data='Tint2 Tint1 Text2 Text1 T-S T-B T-C',dlim={yrange:[-80,70],ystyle:1,ytitle:'Temperature (C)'}

;   tplot_names
  ; tplot,'I_Vreg V_R T_all I_Heater V_max V_atten V_mux'


   store_data,'AO0',data={x:dap.time,y:dap.data[16]}
   store_data,'AO1',data={x:dap.time,y:dap.data[17]}

end
